/*
 * Frequency-response matrix object
 *
 * Copyright (c) Solexa   2005, 2006
 *               Illumina 2007
 * Author:
 *           Klaus Maisinger
 * This source file is covered by the "Illumina Genome Analyzer
 * Software License Agreement" and bound by the terms therein.
 */
#ifndef _MATRIX_H_
#define _MATRIX_H_

class FrequencyMatrix {
private:
    float *m_Matrix;
    int    m_NumChannels;
    char  *m_Bases;
    float *m_ExposureTimes;
    float *m_Power;
protected:
    virtual void getReorderingIndex(char * bases,
                                    int  * index);

public:
    FrequencyMatrix();
    FrequencyMatrix(float * matrix,
                   int     numChannels = 4,
                   char  * bases = 0,
                   float * exposureTimes = 0);
    virtual ~FrequencyMatrix();
    virtual void setMatrix(float * matrix,
                           int     numChannels = 4,
                           char  * bases = 0,
                           float * exposureTimes = 0);
    virtual void identity(int n);
    virtual int read(const char filename[]);
    virtual int write(const char filename[]);
    virtual int write();
    virtual float *getMatrix(char  * bases = 0,
                             float * exposureTimes = 0);
    virtual float *getInverseMatrix(char  * bases = 0,
                                    float * exposureTimes = 0);
    virtual void setPowerCorrection(float * power = 0);
    virtual char getBase(int index) { return m_Bases[index]; }
    virtual int size() { return m_NumChannels; };
};

#endif //_MATRIX_H_
