/*
 * Time-stamp: <$Id: PrbData.h,v 1.3 2008/03/31 12:01:22 km Exp $>
 * 
 * Copyright (c) 2004-2007 Solexa Ltd.
 *               2007      Illumina
 * Author:       Klaus Maisinger, Lukasz Szajkowski
 *
 * This source file is covered by the "Illumina Genome Analyzer
 * Software License Agreement" and bound by the terms therein.
 */
#ifndef PRBDATA_H_
#define PRBDATA_H_

#include <string>
#include <vector>

class PrbData
{
    std::string m_Filename;
public:
    PrbData(const char filename []);
    virtual ~PrbData();
    int write(const std::vector<float * > & probabilities,
              const int                     numClusters,
              const int                     numChannels = 4);
    /* obsolete version */
    int write(const float *prob,
              const int    numClusters,
              const int    numColumns,
              const int    numChannels = 4);
    
    /**
     * The method reads prb data from the filename
     * and parses it to probabilities vector.
     * The method returns number of bytes read.  
     */
    int read(std::vector<float * > & probabilities,
    		 unsigned int          & numClusters,
    		 unsigned int          & numChannels);    
    
    /**
     * The method parses string separated with end of line "\n"
     * from quality file (prb) to vector of quartets of values.
     * The method can be also used for one line parsing
     * 
     * dataSourceName - name of the file (can be empty "")
     * strin - input string
     * probabilities - vector of quartets of probabilities
     * numClusters - number of clasters in the strin string
     * numChannels - number of chanels (at the moment default 4)
     */
    int parsePrb( const std::string     & dataSourceName,
                  const std::string     & strin,
                  std::vector<float * > & probabilities,
                  unsigned int          & numClusters,
                  unsigned int          & numChannels);    
};

int write_quality_file(const char                    filename[],
                       const std::vector<float * > & prob,
                       const int                     nobj,
                       const int                     ncol);
int write_quality_file(const char                    filename[],
                       const float                  *prob,
                       const int                     nobj,
                       const int                     nprob,
                       const int                     ncol);

#endif /*PRBDATA_H_*/
