/*
 * Copyright (c) 2005-2007 Solexa
 * Author: Klaus Maisinger
 *
 * This source file is covered by the "Illumina Genome Analyzer
 * Software License Agreement" and bound by the terms therein.
 */

#ifndef _PROCESSBASECALL_H_
#define _PROCESSBASECALL_H_

#include <vector>
#include "PhasingModel.h"

class ProcessBaseCall
{
    int             m_NumCycles;
    PhasingModel    m_PhasingModel;
    unsigned char * m_BaseOrder;
    int             m_NumChannels;
    int           * m_DeblockOrder;
public:
    ProcessBaseCall(const int numCycles = 1);
    virtual ~ProcessBaseCall();
    void setPhasingModel(const PhasingModel &phasingModel);
    void setCycleNumber(const int numCycles);
    void setBaseOrder(const unsigned char *bases,
                      const int            numChannels = 4);
    void setDeblockingOrder(const int * deblockOrder,
                            const int   numCycles = -1);
    void call(std::vector<float * > & intensities,
              std::vector<float * > & noise,
              std::vector<float * > & matrices,
              const int               numObjects,
              const int               numChannels,
              const int               cycle,
              unsigned char         * sequence,
              float                 * qualityScores,
              float                 * signal,
              float                 * matrixSignal);
    int getWindowSize(const int cycle = 0);
private:
    void applyPhasingCorrection(std::vector<float * > & intensities,
                                std::vector<float * > & noise,
                                std::vector<float * > & matrices,
                                const int               numObjects,
                                const int               numChannels,
                                const int               cycle,
                                float                 * res_signal,
                                float                 * corr_signal, 
                                float                 * qualityScores);
};

#endif //_PROCESSBASECALL_H_
