/*
 * Copyright (c) Illumina 2008
 *
 * This source file is covered by the "Illumina Genome Analyzer
 * Software License Agreement" and bound by the terms therein.
 */
#ifndef IO_UTILS
#define IO_UTILS

#include <string>
#include <iostream>
#include <zlib.h>
#include <errno.h>

#include "Message.h"

namespace IoUtils {

class AutoFile
{
public:
    operator FILE *() {return fh_;}
    AutoFile(const std::string &fileName, const std::string &mode);
    virtual ~AutoFile() { if (fh_) fclose(fh_); }
private:
    FILE *fh_;
};

class AutoGzFile
{
public:
    operator  gzFile() {return fh_;}
    AutoGzFile(const std::string &fileName, const std::string &mode, const unsigned int bufferSize = (1<<16)) throw (std::ios_base::failure);
    virtual ~AutoGzFile() { gzclose(fh_); delete line_; delete buffer_;}
    // read a complete line, excluding EOL and return a copy to the data read.
    std::string getline() throw (std::ios_base::failure);
    // C style 0-terminated string containing the whole line (including '\n')
    char *gets() throw (std::ios_base::failure);
    inline int puts(const char *s) {return gzputs(fh_, s);}
    inline int puts(const std::string &s) {return this->puts(s.c_str());}
    inline int putc(int c) {return gzputc(fh_, c);}
    bool fileError() const {return Z_NULL == fh_;}
    bool eof() const {return gzeof(fh_);}
private:
    gzFile fh_;
    std::string data_;
    char *line_;
    unsigned int lineSize_;
    unsigned int bufferSize_;
    char * buffer_; 
    char *lineStart_;
    char *bufferEnd_;
    char *fillBuffer   (char * const fromPosition) throw (std::ios_base::failure);
    char *findEndOfLine(char * const fromPosition) throw (std::ios_base::failure);
};

} // namespace IoUtils

#endif // #ifndef IO_UTILS
