/*
 * Copyright (c) Solexa 2005, 2006
 * This source file is covered by the "Illumina Genome Analyzer
 * Software License Agreement" and bound by the terms therein.
 */

#ifndef _TILING_H_
#define _TILING_H_

#include <string>
#include <map>


class RunFolderFileName {
    std::string path;
    int *channelidx, *channelcount, nchannel, tileoffset;
public:
    typedef enum {
        POSITION_FILE,
        INDEX_FILE,
        INTENSITY_FILE,
        NOISE_FILE,
        HEADER_FILE,
        OFFSET_FILE,
        CLUSTER_FILE,
        QC_FILE,
        SIGNAL_FILE,
        SEQUENCE_FILE,
        PROBABILITY_FILE,
        SIGNAL2_FILE
    } filename_type;
private:
    std::map<filename_type,std::string> typestring; 

public:
    RunFolderFileName(const char prefix[]);
    ~RunFolderFileName();
    void setLanes(const int *laneIndex, 
                  const int *laneNumTiles,
                  const int  numLanes); 
    void setLane(const int lane,
                 const int tileIndex,
                 const int numTiles = 1);
    std::string getIndexFilename(const filename_type type,
                                 const int idx,
                                 const int cycle = -1,
                                 const int channel = -1);
    std::string getTileFilename(const filename_type type,
                                const int lane,
                                const int tile, 
                                const int cycle = -1,
                                const int channel = -1);
    int getLane(const int idx);
    int getTile(const int idx);
    int getIndex(const int lane,
                 const int tile);
    std::string getFilePrefix();
};

#endif // _TILING_H_
