#!/usr/bin/env perl

use warnings;
use strict;



#PROJECT: ELAND (Inexact Matching Program ALlowing Ambiguity)
#MODULE: ELAND.cpp
#AUTHOR: A. J. Cox
#
# Copyright (c) 2003-2006 Solexa Limited. All rights reserved.
# This source file is covered by the "Solexa Public Source License"
# agreement and bound by the terms therein.


# ELAND_test.pl

# Test harness for ELAND
# Makes a test phiX database using squashGenome
# For read lengths 20,25 and 30:
# - Compiles ELAND
# - Makes a query consisting of 
# i) all exact k-mers, both strands
# ii) all single error variants of above
# iii) all 2-error variants for selected k-mers (too big otherwise!)
# - Runs query
# - Checks query for anomalies 

# TBD: proper check of "N" handling

# All files go in /tmp and are cleaned up afterwards unless 
# -keep is specified on command line

my $version;
my $id;
{
    my $cvs_id='@(#) $Id: ELAND_test.pl,v 1.3 2006/07/27 14:04:33 tc Exp $';
    my $cvs_version='$Revision: 1.3 $';
    # Make CVS tag into a displayable string
    $version=$cvs_version;
    $version=~s/\$//g;
    $version=~s/Revision//;
    $id=$cvs_id;
#    $id=~s/\$//g;
} #scope

sub revComp
{
    my $r=reverse($_[0]);
    $r=~tr/ACGTacgt/TGCAtgca/;
    return  $r;
} #revComp



my $suff=$$;
my $testDir="/tmp/ELAND_test_dir_$suff";
my $testSqu="/tmp/ELAND_test_squ_$suff";
my $testEla="/tmp/ELAND_test_imp_$suff";
my $testDat="/tmp/ELAND_test_dat_$suff";
my $testQry="/tmp/ELAND_test_qry_$suff";
my $testOut="/tmp/ELAND_test_out_$suff.vmf";
my $testOut2="/tmp/ELAND_test_out_$suff.txt";
my $testCom;

print "*** ELAND_test.pl output\n";
print "*** ELAND_test.pl version: $id\n";
print "*** Test run started: ", scalar(localtime), "\n";

print "*** Creating directory $testDir\n";


system("/bin/mkdir $testDir") ==0
    or die "Problem making $testDir : $!";



#die;

system ("/usr/bin/make clean")==0
    or die "Problem making clean : $!";


print "*** Building squashGenome executable\n";

system ("/usr/bin/make squashGenome")==0
    or die "Problem making squashGenome : $!";

system ("/bin/mv squashGenome $testSqu")==0
    or die "Problem moving squashGenome to /tmp: $!";

print "*** Getting ident for squashGenome executable\n";

system ("/usr/bin/ident $testSqu")==0
    or die "Problem getting ident for $testSqu : $!";



#$ENV{"OLIGO_LENGTH"}=23;

print "*** Building test database\n";


open (DAT, ">$testDat")
    || die "Problem opening $testDat: $!";

my $thisLine=<main::DATA>;
#my $thisLine;
my $thisLine2;
my $genomeString;
print DAT $thisLine;


while($thisLine=<main::DATA>)
{
    print DAT $thisLine;
    chomp($thisLine);
    $genomeString.=$thisLine;
} #while

close(DAT);

system ("$testSqu $testDir $testDat")==0
    or die "Problem squashing $testDat into $testDir: $!";


my $ol1;
my $ol2;
my $ol3;
my $ol4;
my $olStore;
my $storePos=-9999;

# Use a default set of read lengths if none are specified. 
# Alternatively, can specify just one or two for a shorter test
@ARGV=(20,23,25,27,30,32) if (@ARGV==0); 

#for my $oligoLength(20,25,30)
for my $oligoLength(@ARGV)
{
    last if ($oligoLength eq "-keep");
#    system ('echo $OLIGO_LENGTH');
#    system ("echo bolox");

    system ("/usr/bin/make clean")==0
        or die "Problem making clean : $!";

    print "*** Building ELAND for oligo length $oligoLength\n";

    # add OLIGO_LENGTH to environment variables
    $ENV{"OLIGO_LENGTH"}=$oligoLength;

    # and specify -e to ensure it is passed to the build process
    system ("/usr/bin/make -e eland")==0
        or die "Problem making eland : $!";
    
    system ("/bin/mv eland $testEla")==0
        or die "Problem moving eland to /tmp: $!";

    print "*** Getting ident for ELAND executable\n";

    system ("/usr/bin/ident $testEla")==0
        or die "Problem getting ident for $testEla : $!";

    print "*** Outputting help info for ELAND executable\n";

    system ("$testEla | cat")==0
        or die "Problem outputting help info for $testEla : $!";


    print "*** Building test query for oligo length $oligoLength\n";
    my $numQueries=0;

    open (QRY, ">$testQry") 
        || die "Problem opening $testQry: $!";
    for (my $i=1; $i<=length($genomeString)-$oligoLength+1;$i++)
    {
        $ol1=substr($genomeString,$i-1,$oligoLength);
        $ol2=revComp($ol1);
        $numQueries+=2;
        # Format of sequence name = >A_B_C_D_E_F
        # A=position in genome
        # B=position of first error, "X" if not present
        # C=position of second error, "X" if not present
        # D=type of first error, "X" if not present
        # E=type of second error, "X" if not present
        # F="F" for forward strand, "R" for reverse

        print QRY ">${i}_X_X_X_X_F\n", $ol1, "\n";
        print QRY ">${i}_X_X_X_X_R\n", $ol2, "\n";


        if (($i%347)==0)
        {
            # Test deletion N handling
            for (my $j=1;$j<=$oligoLength;$j++)
            {
                $ol2=$ol1;
                substr($ol2,$j-1,1)="N";
                print QRY ">${i}_${j}_X_D_X_F\n", $ol2, "\n";
                $ol2=revComp($ol2);
                print QRY ">${i}_${j}_X_D_X_R\n", $ol2, "\n";
                $ol2=$ol1;
                substr($ol2,$j-1,1)=".";
                print QRY ">${i}_${j}_X_d_X_F\n", $ol2, "\n";
                $ol2=revComp($ol2);
                print QRY ">${i}_${j}_X_d_X_R\n", $ol2, "\n";
                $numQueries+=4;

                for (my $k=$j+1;$k<=$oligoLength;$k++)
                {
#                    next if ($k==$j);
                    $ol2=$ol1;
                    substr($ol2,$j-1,1)="N";
                    substr($ol2,$k-1,1)="N";
                    print QRY ">${i}_${j}_${k}_D_D_F\n", $ol2, "\n";
                    $ol2=revComp($ol2);
                    print QRY ">${i}_${j}_${k}_D_D_R\n", $ol2, "\n";
                    $ol2=$ol1;
                    substr($ol2,$j-1,1)=".";
                    substr($ol2,$k-1,1)=".";
                    print QRY ">${i}_${j}_${k}_d_d_F\n", $ol2, "\n";
                    $ol2=revComp($ol2);
                    print QRY ">${i}_${j}_${k}_d_d_R\n", $ol2, "\n";
                    $numQueries+=4;
                } # for k   

            } # for j   

            # Test insertion N handling
            for (my $j=1;$j<=$oligoLength-2;$j++)
            {
                $ol2=substr($ol1,0,$j)."N".substr($ol1,$j,$oligoLength-$j-1);
                die unless (length($ol2)==$oligoLength);
#                substr($ol2,$j-1,1)="N";
                $j++;
                print QRY ">${i}_${j}_X_I_X_F\n", $ol2, "\n";
                $ol2=revComp($ol2);
                print QRY ">${i}_${j}_X_I_X_R\n", $ol2, "\n";
                $j--;
                $ol2=substr($ol1,0,$j).".".substr($ol1,$j,$oligoLength-$j-1);
                $j++;
                print QRY ">${i}_${j}_X_i_X_F\n", $ol2, "\n";
                $ol2=revComp($ol2);
                print QRY ">${i}_${j}_X_i_X_R\n", $ol2, "\n";
                $j++;
                $numQueries+=4;
                for (my $k=$j+1;$k<=$oligoLength-2;$k++)
                {
                    $ol2=substr($ol1,0,$j)
                        ."N".substr($ol1,$j,$k-$j-1)
                        ."N".substr($ol1,$k-1,$oligoLength-$k-1);
                    die unless (length($ol2)==$oligoLength);
                    $j++; $k++;
                    print QRY ">${i}_${j}_${k}_I_I_F\n", $ol2, "\n";
                    $ol2=revComp($ol2);
                    print QRY ">${i}_${j}_${k}_I_I_R\n", $ol2, "\n";
                    $j--; $k--;
                    $numQueries+=2;
                } #  if

            } # for j   

        } # if

        # Now test substitution errors
        for (my $j=1;$j<=$oligoLength;$j++)
        {
            $ol2=$ol1;
            for (my $k=1;$k<=3;$k++)
            {
                substr($ol2,$j-1,1)=~tr/ACGTacgt/CGTAcgta/;
                $ol3=revComp($ol2);
                $numQueries+=2;
                print QRY ">${i}_${j}_X_${k}_X_F\n", $ol2, "\n";
                print QRY ">${i}_${j}_X_${k}_X_R\n", $ol3, "\n";
                if (($i%347)==0)
                {
                    for (my $l=$j+1;$l<=$oligoLength;$l++)
                    {
                        $ol3=$ol2;
                        for (my $m=1;$m<=3;$m++)
                        {
                            substr($ol3,$l-1,1)=~tr/ACGTacgt/CGTAcgta/;
                            $ol4=revComp($ol3);
                            $numQueries+=2;
                            print QRY ">${i}_${j}_${l}_${k}_${m}_F\n", 
                            $ol3, "\n";
                            print QRY ">${i}_${j}_${l}_${k}_${m}_R\n", 
                            $ol4, "\n";
                        } #for m
                    } #for l

                } #if
            } #for k
        } #for j
    } #for
    close(QRY);

    print "*** Generated test query file of $numQueries sequences\n";
    print "*** Running test query for oligo length $oligoLength\n";


    $testCom="$testEla $testQry $testDir $testOut";

    system($testCom) ==0
        or die "Problem running query $testCom: $!";

    # TC 26.01.2006 - test of new output format
    $testCom="$testEla $testQry $testDir $testOut2";

    system($testCom) ==0
        or die "Problem running query $testCom: $!";

    print "*** md5sum of vmf output file:\n";
    system("/usr/bin/md5sum $testOut") ==0
        or die "Problem getting md5sum of $testOut: $!";

    print "*** md5sum of txt output file:\n";
    system("/usr/bin/md5sum $testOut2") ==0
        or die "Problem getting md5sum of $testOut2: $!";
#    die;
    open(RES, "$testOut") 
        || die "Problem opening $testOut : $!";
    open(RES2, "$testOut2") 
        || die "Problem opening $testOut2 : $!";

    my $numRes=0;
    my $numBad=0;
    my @s1;
    my @s2;
    my $isBad;
    my $nType;
    my $nInfo;
    my $warns;
    my $r;
    my $alignRead;
    my $alignNs;
    my $alignSub;

    print "**** Printing any anomalous lines\n";

    while(defined($thisLine=<RES>))
    {
        ++$numRes;
        die "Lines left in $testOut but not $testOut2"
            unless (defined($thisLine2=<RES2>));
        chomp($thisLine);
        chomp($thisLine2);

        @s1=split("\t",$thisLine);
#        print scalar(@s1), "\n";
#        print $thisLine;
#        die;
#        die "bad input line: $thisLine" unless (scalar(@s1)>=7);
        $s1[1]=~s/^>//;
        @s2=split("_",$s1[1]);
#        die "bad input line: $thisLine" unless (scalar(@s2)==5);

        if ($s1[1]=~/[NDdIi\.]/)
        { # then check N handling
            $warns="";
            $isBad=0;
            # if read starts with one N match position increases by 1 
            # if read starts with two Ns match position increases by 2 
            # TBD does this need to be fixed?
          #  $s2[0]++ if ($s1[2]=~/^[N\.]/);
          #  $s2[0]++ if ($s1[2]=~/^[N\.]{2}/);


            if (defined ($s1[10]))
            {
                $nType=uc($s2[3].$s2[4]);
#                $nType=~s/X/\./g;
                if ($s2[2] eq "X")
                {
                    if (($s2[1]==1)||($s2[1]==$oligoLength))
                    {
                        $nType="..";
                    }                        
                    else
                    {
                        $nType=uc($s2[3].".");
                    }
                } # if
                else
                {    
                    if ($s2[1]==1)
                    {
                        if (($s2[2]==2)||($s2[2]==$oligoLength))
                        {
                            $nType="..";
                        } # if                       
                        else 
                        {
                            $nType=uc($s2[4]).".";
                        } # else
                    } # if
                    elsif ($s2[2]==$oligoLength)
                    {
                        if ($s2[1]==$oligoLength-1)
                        {
                            $nType="..";
                        } # if
                        else
                        {
                            $nType=uc($s2[3]).".";
                        } # else
                    } # elsif
                } # else

                if ($nType ne $s1[10])
                {
                    $isBad=1;
                    $warns.= "*** Expecting match type to be $nType\n";
                }

                # Now check match pos
                if (($s2[0] != $s1[8])||($s2[-1] ne $s1[9]))
                {
                    $isBad=1; 
                    $warns.= "*** Match position should be $s2[0] $s2[-1]\n";
                } # if
                
            } # if
            else 
            {
                $isBad=1;
                $warns.= "*** Could not find match type info\n";
            } # else
            if ($isBad==1)
            {
                if (defined($s1[10]))
                {
                    $r=substr($genomeString,$s1[8]-1,$oligoLength);
                    $alignRead=$s1[2];
                    $alignNs=$s1[10];
                    $alignSub="D";

                    # reverse read if necessary to match genome orientation
                    if ($s1[9] eq "R")
                    {
                        $alignRead=reverse($alignRead);
                        $alignRead=~tr/ACGTacgt/TGCAtgca/;
                        $alignNs=reverse($alignNs) unless ($alignNs=~/\./);
                    }
#                    print "$alignRead $alignNs\n";
                    # remove leading blanks
                    $alignRead=~s/^[Nn\.]+//;

                    for (my $i=0;$i<2; $i++)
                    {
                        if (substr($alignNs,$i,1) eq "D")
                        {
                            $alignSub="D";
                        }
                        elsif (substr($alignNs,$i,1) eq "I")
                        {
                            $alignSub="I";
                        }
#                        unless (defined($alignRead)&&defined($alignSub))
#                        {
#                            print "BLX\n$thisLine\n$thisLine2\n";
#                        }
                        $alignRead=~s/[Nn\.]/$alignSub/;
                    }
                    $alignRead=~s/D/./g;
                    $alignRead=~s/I//g;
#                    print "$alignRead\n";
                    $isBad=0 if ($r=~/^$alignRead/);
                } # if

                if ($isBad==1)
                {
                    print "$warns";
                    print "$r\n$alignRead\n$thisLine\n";
                    $numBad++;
                } # if

            } # if isBad
        } # if
        else
        {
            if ($s2[2] eq'X')
            { # either 0 or 1 error
                if ($s2[1]eq 'X')
                { # exact match

                    unless (($s1[3] eq'U0')
                            &&($s1[8]==$s2[0])
                            &&($s1[9] eq $s2[5]))
                    {
                        print "$thisLine\n";
                        ++$numBad;
                    }   #unless                 
                } #if
                else
                { # 1 error match
                    unless (($s1[3] eq'U1')
                            &&($s1[8]==$s2[0])
                            &&($s1[9] eq $s2[5])
                            &&($s2[1]==$s1[11]))
                    {
                        print "$thisLine\n";
                        ++$numBad;
                    }   #unless                 
                } #else
            } #if
            else
            { # 2 error match
                unless (($s1[3] eq'U2')
                        &&($s1[8]==$s2[0])
                        &&($s1[9] eq $s2[5])
                        &&($s2[1]==$s1[11])
                        &&($s2[2]==$s1[12]))
                {
                    print "$thisLine\n";
                    ++$numBad;
                }   #unless                 
            
            } #else

        } # else

        # Test assembly format
        # Don't have to do a full compare - just the bit that differs from vmf

        if ($s2[0]!=$storePos)
        {
            $storePos=$s2[0];
            $olStore=substr($genomeString,$storePos-1,$oligoLength);
        } # if

        @s2=split("\t",$thisLine2);
        for (my $i=10; $i<=11; $i++)
        {
            if (defined($s2[$i]))
            {
                die unless ($s2[$i]=~/([0-9]+)([ACGTacgt])/);
                die unless (substr($olStore,$1-1,1) eq $2 );
 
                $s2[$i]=~s/[ACGT]$//;
            } # if
        } # for


        # now make assembly format line look like vmf format line
        shift(@s1);
        die unless (@s1==@s2);
        die unless ($s2[0]=~s/^>//);
      #  $s2[10]=~s/[ACGT]$// if defined ($s2[10]);
      #  $s2[11]=~s/[ACGT]$// if defined ($s2[11]);

        for (my $i=0;$i<@s1;$i++)
        {
            unless ($s1[$i] eq $s2[$i])
            {
                print "$i\n@s1\n@s2\n";
                die "problem matching line\n${thisLine}\nwith line\n${thisLine2}\n";
            } # unless
        } # for

#        print $thisLine;
                    
    } #while
    close(RES);
    print "*** $numBad anomalous results\n";
    die "Problem: $numQueries queries but $numRes results"
        unless ($numQueries==$numRes);
    die "Lines left in $testOut2 but not $testOut"
        if defined(($thisLine2=<RES2>));
    close (RES2);

} #for $oligoLength

print "*** Test run finished with no problems: ", scalar(localtime), "\n";



unless ((@ARGV>=1)&&($ARGV[-1] eq "-keep"))
{
    for ($testDir, $testSqu, $testEla, $testDat, $testQry, $testOut )
    {
        system("/bin/rm -rf $_")==0
            or die "Problem removing $_ : $!";
        print "Removed directory $_\n";
    } # for
} #unless

__END__
>gi|9626372|ref|NC_001422.1| Coliphage phiX174, complete genome
GAGTTTTATCGCTTCCATGACGCAGAAGTTAACACTTTCGGATATTTCTGATGAGTCGAAAAATTATCTT
GATAAAGCAGGAATTACTACTGCTTGTTTACGAATTAAATCGAAGTGGACTGCTGGCGGAAAATGAGAAA
ATTCGACCTATCCTTGCGCAGCTCGAGAAGCTCTTACTTTGCGACCTTTCGCCATCAACTAACGATTCTG
TCAAAAACTGACGCGTTGGATGAGGAGAAGTGGCTTAATATGCTTGGCACGTTCGTCAAGGACTGGTTTA
GATATGAGTCACATTTTGTTCATGGTAGAGATTCTCTTGTTGACATTTTAAAAGAGCGTGGATTACTATC
TGAGTCCGATGCTGTTCAACCACTAATAGGTAAGAAATCATGAGTCAAGTTACTGAACAATCCGTACGTT
TCCAGACCGCTTTGGCCTCTATTAAGCTCATTCAGGCTTCTGCCGTTTTGGATTTAACCGAAGATGATTT
CGATTTTCTGACGAGTAACAAAGTTTGGATTGCTACTGACCGCTCTCGTGCTCGTCGCTGCGTTGAGGCT
TGCGTTTATGGTACGCTGGACTTTGTGGGATACCCTCGCTTTCCTGCTCCTGTTGAGTTTATTGCTGCCG
TCATTGCTTATTATGTTCATCCCGTCAACATTCAAACGGCCTGTCTCATCATGGAAGGCGCTGAATTTAC
GGAAAACATTATTAATGGCGTCGAGCGTCCGGTTAAAGCCGCTGAATTGTTCGCGTTTACCTTGCGTGTA
CGCGCAGGAAACACTGACGTTCTTACTGACGCAGAAGAAAACGTGCGTCAAAAATTACGTGCGGAAGGAG
TGATGTAATGTCTAAAGGTAAAAAACGTTCTGGCGCTCGCCCTGGTCGTCCGCAGCCGTTGCGAGGTACT
AAAGGCAAGCGTAAAGGCGCTCGTCTTTGGTATGTAGGTGGTCAACAATTTTAATTGCAGGGGCTTCGGC
CCCTTACTTGAGGATAAATTATGTCTAATATTCAAACTGGCGCCGAGCGTATGCCGCATGACCTTTCCCA
TCTTGGCTTCCTTGCTGGTCAGATTGGTCGTCTTATTACCATTTCAACTACTCCGGTTATCGCTGGCGAC
TCCTTCGAGATGGACGCCGTTGGCGCTCTCCGTCTTTCTCCATTGCGTCGTGGCCTTGCTATTGACTCTA
CTGTAGACATTTTTACTTTTTATGTCCCTCATCGTCACGTTTATGGTGAACAGTGGATTAAGTTCATGAA
GGATGGTGTTAATGCCACTCCTCTCCCGACTGTTAACACTACTGGTTATATTGACCATGCCGCTTTTCTT
GGCACGATTAACCCTGATACCAATAAAATCCCTAAGCATTTGTTTCAGGGTTATTTGAATATCTATAACA
ACTATTTTAAAGCGCCGTGGATGCCTGACCGTACCGAGGCTAACCCTAATGAGCTTAATCAAGATGATGC
TCGTTATGGTTTCCGTTGCTGCCATCTCAAAAACATTTGGACTGCTCCGCTTCCTCCTGAGACTGAGCTT
TCTCGCCAAATGACGACTTCTACCACATCTATTGACATTATGGGTCTGCAAGCTGCTTATGCTAATTTGC
ATACTGACCAAGAACGTGATTACTTCATGCAGCGTTACCATGATGTTATTTCTTCATTTGGAGGTAAAAC
CTCTTATGACGCTGACAACCGTCCTTTACTTGTCATGCGCTCTAATCTCTGGGCATCTGGCTATGATGTT
GATGGAACTGACCAAACGTCGTTAGGCCAGTTTTCTGGTCGTGTTCAACAGACCTATAAACATTCTGTGC
CGCGTTTCTTTGTTCCTGAGCATGGCACTATGTTTACTCTTGCGCTTGTTCGTTTTCCGCCTACTGCGAC
TAAAGAGATTCAGTACCTTAACGCTAAAGGTGCTTTGACTTATACCGATATTGCTGGCGACCCTGTTTTG
TATGGCAACTTGCCGCCGCGTGAAATTTCTATGAAGGATGTTTTCCGTTCTGGTGATTCGTCTAAGAAGT
TTAAGATTGCTGAGGGTCAGTGGTATCGTTATGCGCCTTCGTATGTTTCTCCTGCTTATCACCTTCTTGA
AGGCTTCCCATTCATTCAGGAACCGCCTTCTGGTGATTTGCAAGAACGCGTACTTATTCGCCACCATGAT
TATGACCAGTGTTTCCAGTCCGTTCAGTTGTTGCAGTGGAATAGTCAGGTTAAATTTAATGTGACCGTTT
ATCGCAATCTGCCGACCACTCGCGATTCAATCATGACTTCGTGATAAAAGATTGAGTGTGAGGTTATAAC
GCCGAAGCGGTAAAAATTTTAATTTTTGCCGCTGAGGGGTTGACCAAGCGAAGCGCGGTAGGTTTTCTGC
TTAGGAGTTTAATCATGTTTCAGACTTTTATTTCTCGCCATAATTCAAACTTTTTTTCTGATAAGCTGGT
TCTCACTTCTGTTACTCCAGCTTCTTCGGCACCTGTTTTACAGACACCTAAAGCTACATCGTCAACGTTA
TATTTTGATAGTTTGACGGTTAATGCTGGTAATGGTGGTTTTCTTCATTGCATTCAGATGGATACATCTG
TCAACGCCGCTAATCAGGTTGTTTCTGTTGGTGCTGATATTGCTTTTGATGCCGACCCTAAATTTTTTGC
CTGTTTGGTTCGCTTTGAGTCTTCTTCGGTTCCGACTACCCTCCCGACTGCCTATGATGTTTATCCTTTG
AATGGTCGCCATGATGGTGGTTATTATACCGTCAAGGACTGTGTGACTATTGACGTCCTTCCCCGTACGC
CGGGCAATAACGTTTATGTTGGTTTCATGGTTTGGTCTAACTTTACCGCTACTAAATGCCGCGGATTGGT
TTCGCTGAATCAGGTTATTAAAGAGATTATTTGTCTCCAGCCACTTAAGTGAGGTGATTTATGTTTGGTG
CTATTGCTGGCGGTATTGCTTCTGCTCTTGCTGGTGGCGCCATGTCTAAATTGTTTGGAGGCGGTCAAAA
AGCCGCCTCCGGTGGCATTCAAGGTGATGTGCTTGCTACCGATAACAATACTGTAGGCATGGGTGATGCT
GGTATTAAATCTGCCATTCAAGGCTCTAATGTTCCTAACCCTGATGAGGCCGCCCCTAGTTTTGTTTCTG
GTGCTATGGCTAAAGCTGGTAAAGGACTTCTTGAAGGTACGTTGCAGGCTGGCACTTCTGCCGTTTCTGA
TAAGTTGCTTGATTTGGTTGGACTTGGTGGCAAGTCTGCCGCTGATAAAGGAAAGGATACTCGTGATTAT
CTTGCTGCTGCATTTCCTGAGCTTAATGCTTGGGAGCGTGCTGGTGCTGATGCTTCCTCTGCTGGTATGG
TTGACGCCGGATTTGAGAATCAAAAAGAGCTTACTAAAATGCAACTGGACAATCAGAAAGAGATTGCCGA
GATGCAAAATGAGACTCAAAAAGAGATTGCTGGCATTCAGTCGGCGACTTCACGCCAGAATACGAAAGAC
CAGGTATATGCACAAAATGAGATGCTTGCTTATCAACAGAAGGAGTCTACTGCTCGCGTTGCGTCTATTA
TGGAAAACACCAATCTTTCCAAGCAACAGCAGGTTTCCGAGATTATGCGCCAAATGCTTACTCAAGCTCA
AACGGCTGGTCAGTATTTTACCAATGACCAAATCAAAGAAATGACTCGCAAGGTTAGTGCTGAGGTTGAC
TTAGTTCATCAGCAAACGCAGAATCAGCGGTATGGCTCTTCTCATATTGGCGCTACTGCAAAGGATATTT
CTAATGTCGTCACTGATGCTGCTTCTGGTGTGGTTGATATTTTTCATGGTATTGATAAAGCTGTTGCCGA
TACTTGGAACAATTTCTGGAAAGACGGTAAAGCTGATGGTATTGGCTCTAATTTGTCTAGGAAATAACCG
TCAGGATTGACACCCTCCCAATTGTATGTTTTCATGCCTCCAAATCTTGGAGGCTTTTTTATGGTTCGTT
CTTATTACCCTTCTGAATGTCACGCTGATTATTTTGACTTTGAGCGTATCGAGGCTCTTAAACCTGCTAT
TGAGGCTTGTGGCATTTCTACTCTTTCTCAATCCCCAATGCTTGGCTTCCATAAGCAGATGGATAACCGC
ATCAAGCTCTTGGAAGAGATTCTGTCTTTTCGTATGCAGGGCGTTGAGTTCGATAATGGTGATATGTATG
TTGACGGCCATAAGGCTGCTTCTGACGTTCGTGATGAGTTTGTATCTGTTACTGAGAAGTTAATGGATGA
ATTGGCACAATGCTACAATGTGCTCCCCCAACTTGATATTAATAACACTATAGACCACCGCCCCGAAGGG
GACGAAAAATGGTTTTTAGAGAACGAGAAGACGGTTACGCAGTTTTGCCGCAAGCTGGCTGCTGAACGCC
CTCTTAAGGATATTCGCGATGAGTATAATTACCCCAAAAAGAAAGGTATTAAGGATGAGTGTTCAAGATT
GCTGGAGGCCTCCACTATGAAATCGCGTAGAGGCTTTGCTATTCAGCGTTTGATGAATGCAATGCGACAG
GCTCATGCTGATGGTTGGTTTATCGTTTTTGACACTCTCACGTTGGCTGACGACCGATTAGAGGCGTTTT
ATGATAATCCCAATGCTTTGCGTGACTATTTTCGTGATATTGGTCGTATGGTTCTTGCTGCCGAGGGTCG
CAAGGCTAATGATTCACACGCCGACTGCTATCAGTATTTTTGTGTGCCTGAGTATGGTACAGCTAATGGC
CGTCTTCATTTCCATGCGGTGCACTTTATGCGGACACTTCCTACAGGTAGCGTTGACCCTAATTTTGGTC
GTCGGGTACGCAATCGCCGCCAGTTAAATAGCTTGCAAAATACGTGGCCTTATGGTTACAGTATGCCCAT
CGCAGTTCGCTACACGCAGGACGCTTTTTCACGTTCTGGTTGGTTGTGGCCTGTTGATGCTAAAGGTGAG
CCGCTTAAAGCTACCAGTTATATGGCTGTTGGTTTCTATGTGGCTAAATACGTTAACAAAAAGTCAGATA
TGGACCTTGCTGCTAAAGGTCTAGGAGCTAAAGAATGGAACAACTCACTAAAAACCAAGCTGTCGCTACT
TCCCAAGAAGCTGTTCAGAATCAGAATGAGCCGCAACTTCGGGATGAAAATGCTCACAATGACAAATCTG
TCCACGGAGTGCTTAATCCAACTTACCAAGCTGGGTTACGACGCGACGCCGTTCAACCAGATATTGAAGC
AGAACGCAAAAAGAGAGATGAGATTGAGGCTGGGAAAAGTTACTGTAGCCGACGTTTTGGCGGCGCAACC
TGTGACGACAAATCTGCTCAAATTTATGCGCGCTTCGATAAAAATGATTGGCGTATCCAACCTGCA
