/*
 * Add routine for reading flat binary files to the Image class.
 * 
 * Copyright (c) Solexa 2005, 2006
 * Author: Klaus Maisinger
 * This source file is covered by the "Illumina Genome Analyzer
 * Software License Agreement" and bound by the terms therein.
 */
 
#include <iostream>
#include <cstdio>
#include "BinaryImage.h"

static int read_flat_binary_file(const char filename[], unsigned short **image,
                                 int nx, int ny, int bytes, int image_index)
{
    int nread = 0;
    *image  = new unsigned short[nx*ny];
    FILE *datafile = fopen(filename, "rb");
    if (datafile) {
        fseek(datafile, nx*ny*bytes*image_index, SEEK_SET);
        if (bytes == 2)
            nread = fread(*image, sizeof(unsigned short), nx*ny, datafile);
        else {
            unsigned char *data = new unsigned char [bytes*nx*ny];
            nread = fread(data, bytes, nx*ny, datafile);
            int x, y;
            for (x = 0; x < nx; x++)
                for (y = 0; y < ny; y++)
                    (*image)[x*ny+y] = data[y*nx+x];
        }
        fclose(datafile);
    }
    return nread;
}


BinaryImage::BinaryImage() : Image()
{
}

BinaryImage::~BinaryImage()
{
}

void BinaryImage::LoadBinaryImage(int bytes, int nxa, int nya, float f)
{
    nx = nxa; ny = nya;
    properties->BitsPerPixel = (unsigned short)(8 * bytes);
    snprintf(properties->ImageDescription, sizeof(properties->ImageDescription),
             "Raw image\n");
    std::cout << "bits " << properties->BitsPerPixel;
    image = new unsigned short[nx*ny];
    read_flat_binary_file(get_filename().c_str(), &image, nx, ny, 
                          bytes, get_frame_index());
    params.ClusterFwhm = f;
}
