/*
 * Definition of standard objects
 *
 * Copyright (c) Solexa 2005, 2006
 * Author:
 * Klaus Maisinger
 * This source file is covered by the "Illumina Genome Analyzer
 * Software License Agreement" and bound by the terms therein.
 */
#ifndef _CLUSTEROBJECTS_H_
#define _CLUSTEROBJECTS_H_

#include "ImageUtil.h"

struct ClusterObject {
    struct st_pixel    * boundary;
    int                  nb;
    struct st_pixel    * area;
    int                  na;
    struct st_pixel      max_pixel;
    struct st_position   min, max;
    int                  is_split;
    int                  pass_number;
    float                intensity;
    st_fposition         position;

    // Default Constructor
    ClusterObject();

    void SetObjectBoundary(st_pixel * pBoundary,
                           int nBoundary);

    void SetObjectArea(st_pixel * pArea,
                       int nArea);

    /***************************************************************************
    * Shift all coordinate references by (x0, y0)
    ****************************************************************************/
    void Shift(unsigned short x0,
               unsigned short y0);

    /***************************************************************************
    * Create the smallest 2-d image that contains the pixels given
    * for an object. All other pixels values are set to <default_value>
    ****************************************************************************/
    float * MakeNewImage(int *nx, int *ny, float default_value = 0.0f);

    /***************************************************************************
    * Debugging aid: write out image area as a table of pixel values
    ****************************************************************************/
    void Display();

    void clear();
};

#endif //_CLUSTEROBJECTS_H

