/**********************************************************************************
File:        BackEstimator.cpp
Description: Include file for the CBackEstimator class. 
Created:     12/12/07  SBanerjee

Copyright (c) 2006-2010, Solexa Inc, All rights reserved.
This information is protected by copyright law and international
treaties.  Unauthorized reproduction or distribution of this file, or
any portion of it, may result in severe civil and criminal penalties,
and will be prosecuted to the maximum extent possible under the law.
**********************************************************************************/

#ifndef BACK_ESTIMATOR_H
#define BACK_ESTIMATOR_H

#include "SmartImage.h"

class CBackEstimator
{
public:
    /***********************************************************************************************
    Function:    Constructor
    Description: Creates an instance of CBackEstimator.
    Returns:     None
    Arguments:   None
    Globals:     None
    ***********************************************************************************************/
    CBackEstimator()
        : m_iAvgBoxSize(0)
    {}

    /***********************************************************************************************
    Function:    Destructor
    Returns:     None
    Arguments:   None
    Globals:     None
    ***********************************************************************************************/
    virtual ~CBackEstimator() {}

    /***********************************************************************************************
    Function:    Setup
    Description: Sets it up with the initial image.
    Returns:     Status / Error code 
    Arguments:   Input: The kernel size for box summing.
                 const int iAvgBoxSize:(I) The kernel size for box summing
                 Input: A Single 16-bit monochrome image.
                 const CShortImage & inPicture:(I) Input Picture
    Globals:     None
    ***********************************************************************************************/
    int Setup(const int iAvgBoxSize,
              const CShortImage & inPicture);             

    /***********************************************************************************************
    Function:    Add
    Description: Adds the foreground picture to update the swiss cheese image.
    Returns:     None
    Arguments:   Input: A Foreground Picture
                 const CGrayImage & forePicture:(I) Foreground Picture
    Globals:     None
    ***********************************************************************************************/
    void Add(const CGrayImage & forePicture);

    /***********************************************************************************************
    Function:    Run
    Description: Runs the CBackEstimator to create the background image.
    Returns:     None
    Arguments:   None
    Globals:     None
    ***********************************************************************************************/
    void Run();

    /***********************************************************************************************
    Function:    GetBackMeanPicture
    Description: Accessor to the background mean picture
    Returns:     Returns the background mean picture
    Arguments:   None
    Globals:     None
    ***********************************************************************************************/
    const CFloatImage & GetBackMeanPicture() const;

    /***********************************************************************************************
    Function:    GetBackVarPicture
    Description: Accessor to the background variance picture.
                 Note that the variance image is NOT the true variance image. To create the
                 true variance image, the square of the mean has to be subtracted from this. This
                 subtraction is NOT carried out here. It is done in the last step in method
                 IAMDetector::updateAnalysisImages() for reasons of computational efficiency.
    Returns:     Returns the background "variance" picture.
    Arguments:   None
    Globals:     None
    ***********************************************************************************************/
    const CFloatImage & GetBackVarPicture() const;

private:
    int         m_iAvgBoxSize;
    CGrayImage  m_SwissCheese;
    CUIntImage  m_NumPicture;
    CFloatImage m_BackMeanPicture;
    CFloatImage m_BackVarPicture;
};

/* ******* */
/* Inlines */
/* ******* */
inline const CFloatImage & CBackEstimator::GetBackMeanPicture() const {return m_BackMeanPicture;}
inline const CFloatImage & CBackEstimator::GetBackVarPicture() const {return m_BackVarPicture;}

#endif  // BACK_ESTIMATOR_H


