/**********************************************************************************
File:        Convoluter.cpp
Description: Implementation for the convolution with separable filters
             using Intel library.
Created:     02/09/07  Saibal Banerjee

Copyright (c) 2007-2017, Illumina, All rights reserved.
This information is protected by copyright law and international
treaties.  Unauthorized reproduction or distribution of this file, or
any portion of it, may result in severe civil and criminal penalties,
and will be prosecuted to the maximum extent possible under the law.
**********************************************************************************/
#include <stdlib.h>
#include <string.h>
#include "ErrorCodes.h"
#include "Convoluter.h"

static void RunFloat1D(register int     iArraySize,
                       register float * pArray,
                       register int     iFilterSize,
                       register float * pFilter,
                       register float * pResult)
{
    register float fSum;
    register int   iLength;
    register int   iBump = iFilterSize - 1;

    pResult += iBump;

    iArraySize -= iBump;
    while (iArraySize--)
    {
        fSum = (*pFilter++) * (*pArray++);
        iLength = iBump;
        while (iLength--) fSum += (*pFilter++) * (*pArray++);
        *pResult++ = fSum;
        pFilter -= iFilterSize;
        pArray -= iBump;
    }
}

int CConvoluter::Run(int     iPicWidth,
                     int     iPicHeight,
                     float * pPicture,
                     float * pXFilter,
                     int     iFilterWidth,
                     float * pYFilter,
                     int     iFilterHeight)
{
    int iStatus = OKAY;
    int iHt, iWd;
    float * pBuf;

    int iPicSize = iPicWidth * iPicHeight;
    int iKerSize = (iFilterWidth >= iFilterHeight) ? iFilterWidth : iFilterHeight;
    float * pScratch = new float [iPicSize + iKerSize - 1];
    if (!pScratch)
    {
        iStatus = MEMORY_ALLOCATION_FAILURE;
        return iStatus;
    }

    // Horizontal convolution
    RunFloat1D(iPicSize, pPicture, iFilterWidth, pXFilter, pScratch);

    // Swap iPicWidth and iPicHeight and transpose
    iHt = iPicHeight;
    iPicHeight = iPicWidth;
    iPicWidth = iHt;

    iWd = iPicWidth;
    while (iWd--)
    {
        pBuf = pPicture++;
        iHt = iPicHeight;
        while (iHt--)
        {
            *pBuf = *pScratch++;
            pBuf += iPicWidth;
        }
    }
    pScratch -= iPicSize;
    pPicture -= iPicWidth;

    // Vertical convolution
    RunFloat1D(iPicSize, pPicture, iFilterHeight, pYFilter, pScratch);

    // Swap iPicWidth and iPicHeight and transpose
    iHt = iPicHeight;
    iPicHeight = iPicWidth;
    iPicWidth = iHt;

    iWd = iPicWidth;
    while (iWd--)
    {
        pBuf = pPicture++;
        iHt = iPicHeight;
        while (iHt--)
        {
            *pBuf = *pScratch++;
            pBuf += iPicWidth;
        }
    }
    pScratch -= iPicSize;
    pPicture -= iPicWidth;

    delete [] pScratch;
    return iStatus;
}

