/****************************************************************************
File:        GaussianFilter.cpp
Description: Implementation file for Gaussian filters.
Created:     12/17/03  SBanerjee

Copyright (c) 1996-2006, Saibal Banerjee, All rights reserved.
This information is protected by copyright law and international
treaties.  Unauthorized reproduction or distribution of this file, or
any portion of it, may result in severe civil and criminal penalties,
and will be prosecuted to the maximum extent possible under the law.
****************************************************************************/
#if defined(__BORLANDC__)
#include <math.h>
#include <limits.h>
#else
#include <cmath>
#include <climits>
#endif
#include "GaussianFilter.h"

CGaussianFilter::CGaussianFilter(const float fSigma)
    : m_fSigma(fSigma)
    , m_iSize(0)
    , m_pBlur(0)
{
    if (m_fSigma <= 0)
    {
        return;
    }

	m_iSize = (int) (3.0 * m_fSigma + 0.5);
	m_iSize = 2 * m_iSize + 1;

    m_pBlur = new float [m_iSize];
    if (!m_pBlur)
    {
        m_iSize = 0;
        m_fSigma = 0;
        return;
    }

    const int iLimit = m_iSize >> 1;
    double dVal; 
    float fBlurSum = 0;
    int index;

    for (index = -iLimit; index <= iLimit; index++)
    {
        dVal = index / m_fSigma;
        dVal *= dVal;
        dVal /= 2.0;
        m_pBlur[index + iLimit] = (float) exp(-dVal);
        fBlurSum += m_pBlur[index + iLimit];
    }

    for (index = 0; index < m_iSize; index++)
    {
        m_pBlur[index] /= fBlurSum;
    }
}

CGaussianFilter::~CGaussianFilter()
{
    if (m_pBlur) delete [] m_pBlur;

    m_pBlur = 0;
    m_iSize = 0;
    m_fSigma = 0;
}

float CGaussianFilter::Get2DBlurSumOfSquares() const
{
    float fSumSq = 0;
    for (int i = 0; i < m_iSize; i++)
    {
        fSumSq += m_pBlur[i] * m_pBlur[i];
    }
    return fSumSq * fSumSq;
}
