/****************************************************************************
File:        GaussianMaker.h
Description: Include file for Gaussian filter maker.
Created:     12/17/03  SBanerjee

Copyright (c) 1996-2006, Saibal Banerjee, All rights reserved.
This information is protected by copyright law and international
treaties.  Unauthorized reproduction or distribution of this file, or
any portion of it, may result in severe civil and criminal penalties,
and will be prosecuted to the maximum extent possible under the law.
****************************************************************************/

#ifndef GAUSSIANMAKER_H
#define GAUSSIANMAKER_H

class CGaussianMaker
{
public:
    // Constructor
    CGaussianMaker()
        : m_dSigma(0)
        , m_dXCenter(0)
        , m_dYCenter(0)
        , m_dIntensity(0)
    {};

    // Copy Constructor
    CGaussianMaker(const CGaussianMaker & source);

    // Assignment: from CGaussianMaker to CGaussianMaker
    CGaussianMaker & operator = (const CGaussianMaker & source);

    // Set Function: Modifier
    void Set(const float fSigma,
             const float fXCenter,
             const float fYCenter,
             const float fIntensity);

    // GetValue Function
    double GetValue(const int iX,
                    const int iY) const;

    // GetROI Function
    void GetROI(int & iROIXStart,
                int & iROIYStart,
                int & iROIWidth,
                int & iROIHeight) const;

    // GetCenter Function
    void GetCenter(float & fXCenter,
                   float & fYCenter) const;

private:
    static const double msc_dCeilAdd;
    double              m_dSigma;
    double              m_dXCenter;
    double              m_dYCenter;
    double              m_dIntensity;
};

#endif // GAUSSIANMAKER_H
