/**********************************************************************************
File:        Matcher.h
Description: Include file for the CMatcher class. 
Created:     03/01/07  SBanerjee

Copyright (c) 2006-2010, Solexa Inc, All rights reserved.
This information is protected by copyright law and international
treaties.  Unauthorized reproduction or distribution of this file, or
any portion of it, may result in severe civil and criminal penalties,
and will be prosecuted to the maximum extent possible under the law.
**********************************************************************************/
#ifndef MATCHER_H
#define MATCHER_H

#include "GaussianFilter.h"
#include "SmartImage.h"

class CMatcher
{
public:
    // Default Constructor
    CMatcher()
        : m_fCorrThresh(0.5f)
        , m_iStatus(OKAY)
    {};

    // Set Functions: Modifiers
    void SetCorrelationThreshold(const float fCorrThresh);

    // Run Function
    int Run(const CGaussianFilter & filter,
            const CShortImage     & picture);

    // CrossCorrelate Function
    int CrossCorrelate(const CGaussianFilter & filter,
                       const int               iPicWidth,
                       const int               iPicHeight,
                       const unsigned short *  pPicBuffer);

    // NonMaxSuppress Function
    int NonMaxSuppress(const int              iPicWidth,
                       const int              iPicHeight,
                       const unsigned short * pPicBuffer);

    // Destructor
    virtual ~CMatcher() {};

	// Accessors
    const CFloatImage & GetMatchPicture() const;
    const CGrayImage &  GetForePicture()  const;
    const CGrayImage &  GetPeakPicture()  const;
    const int           GetStatus()       const;

private:
    float             m_fCorrThresh;

    CFloatImage       m_MatchPicture;
    CGrayImage        m_ForePicture;
    CGrayImage        m_PeakPicture;

    CFloatImage       m_SumPicture;
    CFloatImage       m_SumOfSqPicture;
    CFloatImage       m_ScratchPicture;
    CShortImage       m_DilatePicture;

    int               m_iStatus;
};

/* ******* */
/* Inlines */
/* ******* */
inline const CFloatImage & CMatcher::GetMatchPicture() const {return m_MatchPicture;}
inline const CGrayImage &  CMatcher::GetForePicture()  const {return m_ForePicture;}
inline const CGrayImage &  CMatcher::GetPeakPicture()  const {return m_PeakPicture;}

#endif  // MATCHER_H
