/**********************************************************************************
File:        ObjectDetector.h
Description: Include file for the CObjectDetector class. 
Created:     08/07/06  SBanerjee

Copyright (c) 2006-2010, Solexa Inc, All rights reserved.
This information is protected by copyright law and international
treaties.  Unauthorized reproduction or distribution of this file, or
any portion of it, may result in severe civil and criminal penalties,
and will be prosecuted to the maximum extent possible under the law.
**********************************************************************************/

#ifndef OBJECT_DETECTOR_H
#define OBJECT_DETECTOR_H

#include "SmartImage.h"
#include "GaussianFilter.h"
#include "Matcher.h"
#include "Filterer.h"
#include "BlobLabeler.h"

class CObjectDetector
{
public:
    /***********************************************************************************************
    Function:    Constructor
    Description: Creates an instance of CObjectDetector.
    Returns:     None
    Arguments:   None
    Globals:     None
    ***********************************************************************************************/
    CObjectDetector()
        : m_iStatus(OKAY)
    {};

    /***********************************************************************************************
    Function:    Destructor
    Returns:     None
    Arguments:   None
    Globals:     None
    ***********************************************************************************************/
    virtual ~CObjectDetector() {};

    /***********************************************************************************************
    Function:    Compute
    Description: Performs calculations on an image.
    Returns:     Status / Error code 
    Arguments:   Input: Pointer to a Gaussian Filter.
                 const CGaussianFilter * pFilter     :(I) Ptr to filter.
                 Input: A Single 16-bit monochrome image.
                 const int iImageWidth               :(I) Width of image.
                 const int iImageHeight              :(I) Height of image.
                 const unsigned short * pImageBuffer :(I) Pointer to the image buffer.
    Globals:     None
    ***********************************************************************************************/
    int Compute(const CGaussianFilter * pFilter,
                const float             fCorrelationThreshold,
                const int               iImageWidth,
                const int               iImageHeight,
                const unsigned short *  pImageBuffer);

    void ApplyTransform(const int nFeatIndex,
                        const float fSlope,
                        const float fAbscissa);

    /***********************************************************************************************
    Function:    GetPicture
    Description: Accessor to the picture
    Returns:     Returns the picture
    Arguments:   None
    Globals:     None
    ***********************************************************************************************/
    const CShortImage & GetPicture() const;

    /***********************************************************************************************
    Function:    GetLabeler
    Description: Accessor to the labeler
    Returns:     Returns the labeler
    Arguments:   None
    Globals:     None
    ***********************************************************************************************/
    const CBlobLabeler & GetLabeler() const;

    /***********************************************************************************************
    Function:    GetMatcher
    Description: Accessor to the matcher
    Returns:     Returns the matcher
    Arguments:   None
    Globals:     None
    ***********************************************************************************************/
    const CMatcher & GetMatcher() const;

    /***********************************************************************************************
    Function:    GetFloatFeatures
    Description: Accessor to the feature
    Returns:     Returns the feature
    Arguments:   None
    Globals:     None
    ***********************************************************************************************/
    const CFloatImage & GetFloatFeatures() const;

private:
    int            m_iStatus;

    CShortImage    m_InPicture;
    CShortImage    m_ProcPicture;
    CGrayImage     m_SymbolPicture;
    CMatcher       m_Matcher;
    CFilterer      m_Filterer;
    CBlobLabeler   m_Labeler;

    CFloatImage    m_FltFeatures;
    CUIntImage     m_IntFeatures;

    void featureExtract();
};

/* ******* */
/* Inlines */
/* ******* */
inline const CShortImage & CObjectDetector::GetPicture() const {return m_InPicture;}
inline const CMatcher & CObjectDetector::GetMatcher() const {return m_Matcher;}
inline const CBlobLabeler & CObjectDetector::GetLabeler() const {return m_Labeler;}
inline const CFloatImage & CObjectDetector::GetFloatFeatures() const {return m_FltFeatures;}

#endif  // OBJECT_DETECTOR_H
