/****************************************************************************
File:        Registrationer.h
Description: Include file for image registration.
Created:     03/01/07  SBanerjee

Copyright (c) 2006-2010, Solexa Inc, All rights reserved.
This information is protected by copyright law and international
treaties.  Unauthorized reproduction or distribution of this file, or
any portion of it, may result in severe civil and criminal penalties,
and will be prosecuted to the maximum extent possible under the law.
**********************************************************************************/

#ifndef REGISTRATIONER_H
#define REGISTRATIONER_H

#include "SmartImage.h"

class CRegistrationer
{
public:
    // Constructor
    CRegistrationer()
        : m_iXPriorMove(0)
        , m_iYPriorMove(0)
        , m_iXUncertainty(0)
        , m_iYUncertainty(0)
        , m_dXDelta(0.0)
        , m_dYDelta(0.0)
        , m_dMeasure(0.0)
        , m_iStatus(OKAY)
    {};

    // Run
    int Run(const CFloatImage & fixPicture,
            const CFloatImage & movPicture,
            const int           iXPriorMove,
            const int           iYPriorMove,
            const int           iXMaxUncertainty,
            const int           iYMaxUncertainty);

    // Destructor
    virtual ~CRegistrationer() {};

    // Accessors
    const float GetXAbscissa() const;
    const float GetXSlope() const;
    const float GetYAbscissa() const;
    const float GetYSlope() const;

private:
    static const int msc_iWidth;
    static const int msc_iHeight;
    static const int m_iXDefaultUncertainty;
    static const int m_iYDefaultUncertainty;

    int              m_iXPriorMove;
    int              m_iYPriorMove;
    int              m_iXUncertainty;
    int              m_iYUncertainty;

    CFloatImage      m_UPicture;
    CFloatImage      m_VPicture;
    CDoubleImage     m_Kernel;

    double           m_dXDelta; // Add m_dXDelta to the movable image to register it with the fixed image @ compute
    double           m_dYDelta; // Add m_dYDelta to the movable image to register it with the fixed image @ compute
    double           m_dMeasure;
    int              m_iStatus;

    // Apply m_fXAbscissa and m_fXSlope to the xcoordinates of moveble image to register it with the fixed image
    // Apply m_fYAbscissa and m_fYSlope to the ycoordinates of moveble image to register it with the fixed image
    float            m_fXAbscissa;
    float            m_fXSlope;
    float            m_fYAbscissa;
    float            m_fYSlope;

    void compute(const CFloatImage & fixPicture,
                 const CFloatImage & movPicture,
                 const int           iXROICenter,
                 const int           iYROICenter);

    void copyToUPicture(const float * pSource,
                        const int     nImppl);

    void copyToVPicture(const float * pSource,
                        const int     nImppl);

    double getMatch(const float * pUBuf);

    void getDeltas();

    const double parabolicInterpolate(int      pt,
                                      int      imppl,
                                      double * viz) const;

    void getStatLinearCoeffs(const int n,
                             const double * x,
                             const double * y,
                             float & fSlope,
                             float & fAbscissa);
};

/* ******* */
/* Inlines */
/* ******* */
inline const float CRegistrationer::GetXAbscissa() const {return m_fXAbscissa;}
inline const float CRegistrationer::GetXSlope()    const {return m_fXSlope;}
inline const float CRegistrationer::GetYAbscissa() const {return m_fYAbscissa;}
inline const float CRegistrationer::GetYSlope()    const {return m_fYSlope;}

#endif // REGISTRATIONER_H
