/****************************************************************************
File:        Sombrero.cpp
Description: Implementation file for Mexican Hat filters.
Created:     02/09/07  Saibal Banerjee

Copyright (c) 2007-2017, Illumina, All rights reserved.
This information is protected by copyright law and international
treaties.  Unauthorized reproduction or distribution of this file, or
any portion of it, may result in severe civil and criminal penalties,
and will be prosecuted to the maximum extent possible under the law.
****************************************************************************/
#if defined(__BORLANDC__)
#include <math.h>
#include <limits.h>
#else
#include <cmath>
#include <climits>
#endif
#include "Sombrero.h"

CSombrero::CSombrero(const float fSigma)
    : m_fSigma(fSigma)
    , m_iSize(0)
    , m_pAdd(0)
    , m_pSub(0)
{
    if (m_fSigma <= 0)
    {
        return;
    }

	m_iSize = (int) (3.0 * sqrt(2.0) * m_fSigma);
	m_iSize = 2 * m_iSize + 1;

    m_pAdd = new float [m_iSize];
    if (!m_pAdd)
    {
        deallocate();
        return;
    }
    m_pSub = new float [m_iSize];
    if (!m_pSub)
    {
        deallocate();
        return;
    }

    const int iLimit = m_iSize >> 1;
    double dVal, dExpVal;
    int index;

    for (index = -iLimit; index <= iLimit; index++)
    {
        dVal = index / m_fSigma;
        dVal *= dVal;
        dVal /= 2.0;
        dExpVal = exp(-dVal);

        m_pAdd[index + iLimit] = (float) dExpVal;
        m_pSub[index + iLimit] = (float) (dVal * dExpVal);
        m_pAdd[index + iLimit] -= m_pSub[index + iLimit];
    }
}

void CSombrero::deallocate()
{
    m_fSigma = 0;
    m_iSize = 0;

    if (m_pAdd)
    {
        delete [] m_pAdd;
        m_pAdd = 0;
    }

    if (m_pSub)
    {
        delete [] m_pSub;
        m_pSub = 0;
    }
}

CSombrero::~CSombrero()
{
    deallocate();
}

////////////////////////////////////////////////////////////////////
