/****************************************************************************
File:        IAMDetector.h
Description: Include file for detecting objects with normailzed cross
             correlation based matching and Mexican hat filtering
Created:     10/15/07 
Author:      Saibal Banerjee

Copyright (c) Illumina 2007, All rights reserved.
This information is protected by copyright law and international
treaties.  Unauthorized reproduction or distribution of this file, or
any portion of it, may result in severe civil and criminal penalties,
and will be prosecuted to the maximum extent possible under the law.
****************************************************************************/

#ifndef IAMDETECTOR_H
#define IAMDETECTOR_H

#include "ClusterObject.h"
#include "ObjectDetectorIFC.h"

#include "IAMDetection/SmartImage.h"
#include "IAMDetection/BackEstimator.h"

class CGaussianMaker;
class CObjectDetector;

class IAMDetector : public IObjectDetector
{
public:
    IAMDetector()
        : IObjectDetector()
    {}

    void Run(int                  imageWidth,
             int                  imageHeight,
             float *              imageBuffers[],
             AnalysisParameters & params,
             int &                nObjects,
             ClusterObject *    & pObjects);

private:
    CBackEstimator m_BackEstimator;

    void transposeImage(float *       image,
                        CShortImage & inPicture);

    void makeSubtractor(const unsigned int nClusters,
                        CGaussianMaker *   clusters,
                        CDoubleImage &     picture);

    void nextProcess(const CObjectDetector & detector,
                     const float             fFilterSigma,
                     CShortImage &           inPicture);

    void fillObjectStruct(const CObjectDetector * pDetectors,
                          const int               iWidth,
                          const int               iHeight,
                          const int               iPassNumber,
                          const int               nObjects,
                          ClusterObject *         pObjects);

    void updateAnalysisImages(float * imageBuffers[]);
};

#endif //IAMDETECTOR_H
