/*  Time-stamp: <26 Jun 2006 16:52 BST ImageUtil.h km> */
/*
 * Copyright (c) Solexa 2005,2006
 * Author: Klaus Maisinger
 * This source file is covered by the "Illumina Genome Analyzer
 * Software License Agreement" and bound by the terms therein.
 */

//---------------------------------------------------------------------------

#ifndef _IMAGE_UTIL_H
#define _IMAGE_UTIL_H

#include "ParameterTypes.h"

// The following two definitions help to allocate sensible array sizes
// for internal scratch storage. If these numbers are too high, memory
// is wasted. If they are too low, memory may have to be reallocated,
// or clusters may get dropped.
#define DEFAULT_PIXEL_DIM 2048  // used for allocating sensible default
                                 // array sizes
#define MIN_PIXELS_PER_OBJECT 15  // this used for allocation of arrays 
#define MAX_OBJECTS ((DEFAULT_PIXEL_DIM * DEFAULT_PIXEL_DIM / MIN_PIXELS_PER_OBJECT))

struct st_position {
    unsigned short x;
    unsigned short y;
};

struct st_pixel : public st_position {
    float value;
};

struct st_fposition {
    float x;
    float y;
};

struct st_fpixel {
    float x;
    float y;
    float value;
};

struct st_fnoisepixel {
    float x;
    float y;
    float value;
    float noise;
};

st_fposition *fposition_from_fnoisepixel(st_fnoisepixel *pixel, int n);
st_fposition *fposition_from_pixel(st_pixel *pixel, int n);
st_pixel *pixel_from_fposition(st_fposition *fpos, int npos);
st_pixel *pixel_from_fnoisepixel(st_fnoisepixel *fpos, int npos);
st_fnoisepixel *fnoisepixel_from_pixel(st_pixel *pixel, int n,
                                       float *noise_image = 0, 
                                       int nx = 0, int ny = 0);
st_fnoisepixel *fnoisepixel_from_position(st_fposition *pixel, int n,
                                          float *image = 0, 
                                          float *noise_image = 0,
                                          int nx = 0, int ny = 0);

int write_positions(const char filename[], st_fposition *positions, int n);
int read_positions(const char filename[], st_fposition **positions, int *npos);

void convert_image_uchar2ushort(unsigned char cameradata[],
                                unsigned short image[],
                                int nx, int ny, int tanspose = 0);
void convert_image_ushort2uchar(unsigned short image[],
                                unsigned char cameradata[],
                                int nx, int ny, int transpose = 0);
void convert_image_ushort2float(unsigned short image[], float *fimage,
                                int nx, int ny);
void transpose_ushort_image(unsigned short *image,
                            unsigned short *transpose, int nx, int ny);

const int LOCAL_SIZE = 125;
int get_local_image_size(int ntotal, int nmin);

float get_saturation(float *array, int n, unsigned short nbits);
float get_saturation_fraction(float *array, int n, unsigned short nbits);
float get_saturation(unsigned short *array, int n, unsigned short nbits);
void remove_saturated_pixels(unsigned short *array, int nx, int ny,
                             unsigned short nbits);                               

void copy_image_region(float *image, int nx, int ny,
                       float *region, int sx, int sy, int x = 0, int y = 0);
void add_image_region(float *image, int nx, int ny,
                      float *region, int sx, int sy, 
                      int xpos = 0, int ypos = 0, float amp = 1.);
void add_image_region_constant(float *image, int nx, int ny,
                               int x, int y, int sizex, int sizey,
                               float offset);
void set_image_region_value(float *image, int nx, int ny,
                            int x, int y, int sizex, int sizey,
                            float value);                              
void insert_image_region(float *image, int nx, int ny,
                         float *region, int sx, int sy, 
                         int xpos = 0, int ypos = 0);
void insert_image_region(float *image, int nx, int ny,
                         unsigned short *region, int sx, int sy, 
                         int xpos = 0, int ypos = 0);

// debugging functions
void display_ushort_image_region(unsigned short *image, int nx, int ny,
                                 int x, int y, int nsx, int nsy, 
                                 int offx, int offy);
void display_float_image_region(float *image, int nx, int ny,
                                int x, int y, int nsx, int nsy, 
                                int offx = 0, int offy = 0);

#endif /* _IMAGE_UTIL_H */
