/*
 * Copyright (c) Illumina 2007
 * Author:       Klaus Maisinger
 * This source file is covered by the "Illumina Genome Analyzer
 * Software License Agreement" and bound by the terms therein.
 */
#ifndef MATRIXESTIMATOR_H_
#define MATRIXESTIMATOR_H_

#include <string>

struct MatrixSamplingParameters;

class MatrixEstimator
{
private:
    float       * m_Intensities;
    int           m_NumClusters;
    int           m_NumChannels;
    std::string   m_ErrorMessage;
public:
    MatrixEstimator();
    virtual ~MatrixEstimator();
    void setIntensities(const float * intensities,
                        const int     numClusters, 
                        const int     numChannels = 4);
    //int estimateMatrix(float * matrix);
    int estimateMatrix(float * matrix, 
                       const float *intensities,
                       const int    numClusters, 
                       const int    numChannels = 4);
    void filterIntensities(float       * outputIntensities,
                           int         * outputNumClusters,
                           const float * inputIntensities,
                           const int     inputNumClusters,
                           const int     numChannels);
    int normaliseMatrix(float       * outputMatrix,
                        const float * inputMatrix,
                        const float * intensities,
                        const int     numClusters,
                        const int     numChannels);
    int estimateCrossTalkMatrix(float       * matrix,
                                const MatrixSamplingParameters & params,
                                const float * intensities,
                                const int     numClusters,
                                const int     numChannels);

    const char *getErrorMessage() { return m_ErrorMessage.c_str(); };
    enum ErrorCode {
        errorNone,
        errorNotEnoughClusters,
        errorScalingFailure
    };
};

#endif /*MATRIXESTIMATOR_H_*/
