/****************************************************************************
File:        ObjectDetectorIFC.cpp
Description: Interface for the image processing functions to perform
             Object Detection.
Created:     

Copyright (c) 2007-2017, Illumina Corporation, All rights reserved.
This information is protected by copyright law and international
treaties.  Unauthorized reproduction or distribution of this file, or
any portion of it, may result in severe civil and criminal penalties,
and will be prosecuted to the maximum extent possible under the law.
****************************************************************************/

#include "ObjectDetectorIFC.h"
#include "ProcessObjectDetection.h"
#include "IAMDetector.h"

/****************************************************************************
Function:    CreateInstance
Description: Creates an instance of a concrete class.
Returns:     IObjectDetector * : pointer to created object.
Arguments:   const int iTypeOfConcreteObject
Globals:     None
****************************************************************************/
IObjectDetector * IObjectDetector::CreateInstance(const int iTypeOfConcreteObject)
{
    switch (iTypeOfConcreteObject) {
        case 0:
            return new ClusterDetector();
            break;
        case 1:
            return new IAMDetector();
            break;
        default:
            return 0;
            break;
    }
    return 0;
}
/****************************************************************************
Function:    DeleteInstance
Description: Deletes an instance of a concrete class.
Returns:     None
Arguments:   IObjectDetector * pObject :(I) ref to pointer of created object.
Globals:     None
****************************************************************************/
void IObjectDetector::DeleteInstance(IObjectDetector * & pObject)
{
    if (pObject) delete pObject;
    pObject = 0;
}
