/****************************************************************************
File:        ObjectDetectorIFC.h
Description: Include interface file for the image processing functions to 
             perform Object Detection.
Created:     

Copyright (c) 2007-2017, Illumina Corporation, All rights reserved.
This information is protected by copyright law and international
treaties.  Unauthorized reproduction or distribution of this file, or
any portion of it, may result in severe civil and criminal penalties,
and will be prosecuted to the maximum extent possible under the law.
****************************************************************************/

#ifndef OBJECTDETECTORIFC_H
#define OBJECTDETECTORIFC_H

#include "ClusterObject.h"

class IObjectDetector {
public:
    /****************************************************************************
    Function:    CreateInstance
    Description: Creates an instance of a concrete class.
    Returns:     IObjectDetector * : pointer to created object.
    Arguments:   const int iTypeOfConcreteObject.
    Globals:     None
    ****************************************************************************/
    static IObjectDetector * CreateInstance(const int iTypeOfConcreteObject);

    /****************************************************************************
    Function:    DeleteInstance
    Description: Deletes an instance of a concrete class.
    Returns:     None
    Arguments:   IObjectDetector * & pObject :(I) ref to pointer of created object.
    Globals:     None
    ****************************************************************************/
    static void DeleteInstance(IObjectDetector * & pObject);

    /****************************************************************************
    Function:    Run
    Description: Performs object detection calculations on an image.
    Returns:     None 
    Arguments:   Run
    Globals:     None
    ****************************************************************************/
    virtual void Run(int                  imageWidth,
                     int                  imageHeight,
                     float *              imageBuffers[],
                     AnalysisParameters & params,
                     int &                nObjects,
                     ClusterObject *    & pObjects) = 0;
protected:
    /****************************************************************************
    Function:    Default Constructor
    Arguments:   None
    Globals:     None
    ****************************************************************************/
    IObjectDetector() {};

    /****************************************************************************
    Function:    Destructor
    Arguments:   None
    Returns:     None
    Globals:     None
    ****************************************************************************/
    virtual ~IObjectDetector() {};
};

#endif // OBJECTDETECTORIFC_H

