/*
 * Copyright (c) Solexa 2004-2007
 * Author:
 * Klaus Maisinger
 * This source file is covered by the "Illumina Genome Analyzer
 * Software License Agreement" and bound by the terms therein.
 */

#ifndef ProcessH
#define ProcessH
//---------------------------------------------------------------------------

#include <string>
#include <vector>
#include "ImageProcess.h"
#include "Images.h"
#include "QCRemapping.h"
#include "ProcessAlign.h"
#include "QCWriter.h"
#include "ParameterTypes.h"

const char software[]         = "Firecrest";
const char software_version[] = "1.9.5";

//---------------------------------------------------------------------------

class DataProcessor
{
  private:
    OffsetTracker            *offset_tracker;
    ProcessAlign             *aligner;
    struct RunInfo            run_info;
    struct TileInfo           current_tile;
    struct AnalysisParameters analysis_params;
    int                       max_object_number;
    AlignmentTransform       *default_transforms;
    unsigned char            *bases;
    Image                   **images;
    bool                      useRemappedIntensityFlag;
    bool                      useInterpolatedPositionFlag;
  public:
    DataProcessor(const RunInfo & runInfo);
    ~DataProcessor();
    void SetParameters(const AnalysisParameters & params);
    void SetAlignment(const Alignment * defaultAlignment);
    void SetTile(const TileInfo & tileInfo);
    void AnalyseImage(Image          *image,
                      const int       colour,
                      const char      base,
                      const Position *referencepositions = 0,
                      const int       referencecount = 0);
    void BuildReference(Position       **newReference,
                        int             *numNewReference,
                        const Position  *oldReference = 0,
                        const int        numOldReference = 0);
    void CompleteTile(float       *intensities,
                      float       *noise,
                      Position    *referencePositions,
                      int         &numReference,
                      int          numElements = 0,
                      const char **error_message = 0,
                      int         *flag = 0, 
                      int         *refIndices = 0);
    Image **GetImages() { return images; }
    void displayOffsets();
    void saveOffsets();
    void SetReference(const Position *referencepositions,
                      const int       referencecount);
    void ClearReference();
    // ******************** non-standard interface ****************
    int  validateOffsets();
    void analyseTile(Position *referencepositions,
                     int       referencecount,
                     float    *intensities,
                     float    *noise,
                     int      *refindices = 0);
    void setUseRemappedIntensityFlag(bool value);
};

#endif /* ProcessH */
