/*
 * Time-stamp: <26 Jun 2006 17:01 BST plot_image.cpp km>
 * Copyright (c) Solexa 2004
 * Author: Klaus Maisinger
 * This source file is covered by the "Illumina Genome Analyzer
 * Software License Agreement" and bound by the terms therein.
 */
/*
 * Some high-level image plotting functions based on the PGPLOT graphics 
 * library.
 */
#if defined(HAVE_PGPLOT)
#include <cpgplot.h>
#endif

#include <math.h>
#include "plot.h"
#include "plot_image.h"
#include "Tools.h"
#include "Stats.h"

void plot_image(unsigned short *image, int nx, int ny, float ymin, float ymax)
{
    float *dummy = new float[nx * ny];

    for (int x = 0; x < nx; x++)
        for (int y = 0; y < ny; y++)
            dummy[x * ny + (ny - y - 1)] = (float)image[x * ny + y];
    if (ymin == ymax) {
        ymin = min(dummy, nx * ny);
        ymax = max(dummy, nx * ny);
    }
    plot_2d_array(dummy, nx, ny, 1., 1., 0, ymin, ymax);
    delete[]dummy;
}

void plot_image(float *image, int nx, int ny, float ymin, float ymax)
{
    float *dummy = new float[nx * ny];

    for (int x = 0; x < nx; x++)
        for (int y = 0; y < ny; y++)
            dummy[x * ny + (ny - y - 1)] = (float)image[(x) * ny + (y)];
    if (ymin == ymax) {
        ymin = min(dummy, nx * ny);
        ymax = max(dummy, nx * ny);
    }
    plot_2d_array(dummy, nx, ny, 1., 1., 0, ymin, ymax);
    delete[]dummy;
}

void plot_pixels(st_pixel * positions, int imin, int imax, int nx, int ny)
{
#if defined(HAVE_PGPLOT)
#if 1
    cpgslw(3);
    for (int i = imin; i < imax; i++)
        cpgpt1(positions[i].x - nx / 2, (ny - positions[i].y - 1) - ny / 2,
               -1);
#else
    cpgslw(1);
    cpgsfs(2);
    for (int i = imin; i < imax; i++)
        cpgcirc(positions[3 * i] - n / 2,
                (np - positions[3 * i + 1] - 1) + offs - n / 2, 6.);
#endif
#endif
}

void plot_fpositions(st_fposition * positions, int imin, int imax, int nx,
                     int ny)
{
#if defined(HAVE_PGPLOT)
    cpgslw(3);
    for (int i = imin; i < imax; i++)
        cpgpt1((int)floor(positions[i].x - nx / 2 + 0.5),
               (int)floor((ny - positions[i].y - 1) - ny / 2 + 0.5), -1);
#endif
}
