/*
 * Time-stamp: <$Id: QualityConverter.cpp,v 1.14 2008/04/02 13:28:37 lszajkowski Exp $>
 * File / Format converter module (convert prb file)
 *
 * Copyright (c) Illumina 2007, 2008
 * Author: Lukasz Szajkowski
 * This source file is covered by the "Illumina Genome Analyzer
 * Software License Agreement" and bound by the terms therein.
 */

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <string>
#include <vector>
#include "QualityConverter.h"
#include "Converter.h"
#include "Use_Bases.h"
#include "StringUtil.h"
#include "QrawData.h"
#include "../Bustard/PrbData.h"
#include "../CommonUtils/IOProcess.h"

using namespace std;

QualityConverter::QualityConverter() {
//    _dataSourceNames = NULL;
}

QualityConverter::QualityConverter(const vector<std::string> &filenames) {
    _dataSourceNames = filenames;
}

QualityConverter::~QualityConverter() {
}

int QualityConverter::maxValue(const vector<int> & values) {
    int size = values.size();
    if (size == 0) {
        fprintf(stderr,
        "QualityConverter: empty vector for maxValue method.");
        exit(-1);
    }
    int i;
    int max = values[0];
    for (i = 0; i < size; i++) {
        if (values[i]> max) {
            max = values[i];
        }
    }
    return max;
}

void QualityConverter::toQrawData(const std::vector<float *> & probabilities,
        const unsigned int numClusters, const unsigned int numChannels,
        const vector<unsigned int> & baseIndexTable,
        std::vector<int *> & convertedProbabilities) {
    unsigned int chanelIndex = 0;
    unsigned int clusterIndex;
    unsigned int cycleIndex;
    unsigned int i;

    unsigned int numCycles = probabilities.size();
    unsigned int baseIndexTableSize = baseIndexTable.size();
    unsigned int probabilitiesSize = probabilities.size();
    for (cycleIndex = 0; cycleIndex < baseIndexTableSize; cycleIndex++) {
        chanelIndex = 0;
        unsigned int baseIndex = baseIndexTable[cycleIndex];
        if (baseIndex >= probabilitiesSize){//numCycles) {
            fprintf(stderr,
            "toQrawData: Error processing baseIndexTable.\n");
            fprintf(stderr,
            "toQrawData: Not enouught quality values - %d expected %d\n",
            baseIndex, numCycles + 1);
            exit(-1);
        }

        float * qualityValues = probabilities[baseIndex];
        int * qualityQraw = new int[numClusters];
        for (clusterIndex = 0; clusterIndex < numClusters; clusterIndex++) {
            float maxQuality = qualityValues[chanelIndex];
            for (i = 0; i < numChannels; i++) {
                if (qualityValues[chanelIndex]> maxQuality) {
                    maxQuality = qualityValues[chanelIndex];
                }
                chanelIndex++;
            }
            qualityQraw[clusterIndex] = int (64 + maxQuality);
        }

        convertedProbabilities.push_back(qualityQraw);
    }
}

int QualityConverter::convert(const std::string &qrawFile, const std::string &useBases,
        const int read, const int toFile, const int formatNumeric,
        const int readLength) {

    int status = 0;
    unsigned int numClusters;
    unsigned int numChannels;
    unsigned int fileId;
    vector<unsigned int> baseIndexTable;
    
    parseUseBases(useBases, readLength, baseIndexTable, read);
    QrawData *qrawData = new QrawData(qrawFile.c_str());
    for (fileId = 0; fileId < _dataSourceNames.size(); fileId++) {
        std::vector<float *> probabilities;
        std::string convertedString;
        std::vector<int *> convertedProbabilities;
        std::string convertedStringTmp;
    
        PrbData *prbData = new PrbData(_dataSourceNames[fileId].c_str());
        prbData->read(probabilities, numClusters, numChannels);
        if (probabilities.size() > 0) {
	        toQrawData(probabilities, numClusters, numChannels,
	            baseIndexTable, convertedProbabilities);	
	        qrawData->serialize(convertedProbabilities, numClusters, formatNumeric,
	            convertedStringTmp);
	        convertedString.append(convertedStringTmp);
	      
	        if (toFile == 1) {
	            qrawData->write(convertedString);
	        } else {
	            printf("%s", convertedString.c_str());
	        }
	        vector<float *>::iterator iterProb;
	        for (iterProb = probabilities.begin(); 
	        	iterProb != probabilities.end(); iterProb++) {
	         	delete [] *iterProb;
	        }
	        vector<int *>::iterator iterConvProb;
	        for (iterConvProb = convertedProbabilities.begin(); 
	        	iterConvProb != convertedProbabilities.end(); iterConvProb++) {
	         	delete [] *iterConvProb;
	        }
        }
        delete prbData;
    }
    return status;
}
