#!/usr/bin/env perl

# File: Signal_Means_Monotemplate.pl
# Copyright (c) 2005, 2006 Solexa 
# Author:s A. J. Cox, L. J. Davies
# This source file is covered by the "Solexa Public Source License"
# agreement and bound by the terms therein.

use warnings;
use strict;
use Cwd;

my $version;
my $id;
{
    my $cvs_id='@(#) $Id: Signal_Means_Monotemplates.pl,v 1.11 2007/08/06 08:39:09 rshaw Exp $';
    my $cvs_version='$Revision: 1.11 $';
    # Make CVS tag into a displayable string
    $version=$cvs_version;
    $version=~s/\$//g;
    $version=~s/Revision//;
    $id=$cvs_id;
#    $id=~s/\$//g;
} #scope

# Description:
# Sumamry of template intenstities

my @bases = qw(A C G T);
my $max = 0;
my $setmax = 0;

my ($i,$j,$cycleNum,$sigLine,$alignLine,$perfect,$useBases,$loop,$score,$tag,$pos);
my $sigPipe;
my $alignPipe;
my (@align,@sig,@call,@bcall,@all,@pcall,@tcall,@array);
my %templates;


my $baseCallDir=undef;
my $qhgDir=undef;
my $alignFilePath;
my $sigFilePath;
my $qhgFilePath;


#### Now going to have a summary output in gerald folder??

my $usage=<<"END";
Usage: $0 qualityFilterPath qualityCriterion useBases readLength lane 
[baseCallDir] [qhgDir] sigFile[s] alignFiles[s] qhgFile[s]
baseCallDir - directory where sigFiles are to be found
qhgDir - directory where qhgFiles are to be found
NB in useBases string: N means deblock cycle, n means other non-sequence cycle
END

die $usage unless (@ARGV>=8);


my $qualityFilter=shift(@ARGV);
my $qcCrit=shift(@ARGV);
my $deblock = shift(@ARGV);
my $readLength = shift(@ARGV);
my $lane = shift(@ARGV);
my $numTriplets; #=scalar(@ARGV)/3;


#if(int($numTriplets) != $numTriplets){
#    print STDERR "All triplets are not intact\n";
#    print STDERR "Exiting Now\n";
#    die;
#}


for($i=0;$i<@ARGV;$i++){
    print STDERR "$ARGV[$i]\n";
}

#die;


####Going to handle deblocks here to give a pattern

if($deblock eq 'all'){
    $useBases = 'Y' x $readLength;
}
elsif ($deblock eq 'odd'){
    $useBases = 'YN' x $readLength;
}
else {
    my $numY = 0;
    $useBases = $deblock;
    for($i=0;$i<length($deblock);$i++){
	if(substr($deblock,$i,1) =~ m/[Yy]/){
	    $numY++;
	}
	elsif (substr($deblock,$i,1) !~ m/[Nn]/){
	    die "deblock flag must be only YyNn $deblock\n";
	}
    }
    if($numY != $readLength){
	die "The readLength ($readLength) and deblock usable bases ($numY) do not match\n";
	
    }
 
}
    
print STDERR "$useBases\n";


$score = ($readLength * 500) - (1047 * 4);

if (-d $ARGV[0])
{
    $baseCallDir=shift (@ARGV);
    $baseCallDir.="/" unless ($baseCallDir =~ /\/$/);
} # if

if (-d $ARGV[0])
{
    $qhgDir=shift (@ARGV);
    $qhgDir.="/" unless ($qhgDir =~ /\/$/);
} # if


if ((scalar(@ARGV)%3)!=0)
{
    warn "Expecting triplets of sig,align,qhg files, but num of files (",
    scalar(@ARGV), ") is not a multiple of 3!";
    exit (0);
} # if

$numTriplets=scalar(@ARGV)/3;
print STDERR "Using $numTriplets Triplets\n";


print "#Lane\tCycle\tTemplate\tCount\tAll A\tAll C\tAll G\tAll T\t";
print "Call A\tCall C\tCall G\tCall T\t";
print "Error A\tError C\tError G\tError T\tError .\t";
print "Perfect 0\tPerfect 1\tPerfect 2\tPerfect 3\t";
print "Deblock A\tDeblock C\tDeblock G\tDeblock T\n";

for($loop=0;$loop<$numTriplets;$loop++)
{
    $sigFilePath=$ARGV[$loop];
    $alignFilePath=$ARGV[$numTriplets+$loop];
    $qhgFilePath=$ARGV[$numTriplets+$numTriplets+$loop];
    if (defined($baseCallDir))
    {
        $sigFilePath=$baseCallDir.$sigFilePath;
#        $alignFilePath=$baseCallDir.$seqFilePath;
    } # if
    if (defined($qhgDir))
    {
        $qhgFilePath=$qhgDir.$qhgFilePath;
    } # if

    
    print STDERR "Filtering $sigFilePath AND $alignFilePath with $qhgFilePath\n";

    $sigPipe="cat $sigFilePath |perl $qualityFilter \'$qcCrit\' $qhgFilePath |";
    $alignPipe="cat $alignFilePath |perl $qualityFilter \'$qcCrit\' $qhgFilePath |";
    

    print  STDERR "$sigPipe\n";
    print  STDERR "$alignPipe\n";
    
    
    open (PIPE, $sigPipe) 
        || die "Error: failed to open pipe $sigPipe: $!";
    
    open (PIPE2, $alignPipe) 
        || die "Error: failed to open pipe $alignPipe: $!";
    
    
    
    while (defined($alignLine=<PIPE2>)) {
	next if ($alignLine =~ m/\#/);

	$sigLine = <PIPE> || die "Can't pull in sig Line\n";
	
	@sig = split('\t',$sigLine);
	@align = split(' ',$alignLine);

	next if($score > $align[1]);
	next if($align[2] > 1);
	
	$templates{$align[3]} = $align[5];


	

	## This needs to be back compatible so will not estimate read length

	$cycleNum  = 1;
	$perfect = 0;

	for($j=0;$j<length($useBases);$j++){
	    ## Ignore Noise in older image analysis versions
	    
	    if(substr($useBases,$j,1) =~ m/[Yy]/){
		### Cycle information
		@array = split(' ',$sig[$j+4]);
		

		$all[$align[3]][$cycleNum]{A} += $array[0];
		$all[$align[3]][$cycleNum]{C} += $array[1];
		$all[$align[3]][$cycleNum]{G} += $array[2];
		$all[$align[3]][$cycleNum]{T} += $array[3];
		$all[$align[3]][$cycleNum]{Count}++;


		## Now do the error calls

		##first check if blank
		
		if((substr($align[0],$cycleNum-1,1)) !~ m/[ACGT]/){		    
		    $bcall[$align[3]][$cycleNum]{N}++;
		    
		    $perfect++;
		}
		
		if(substr($align[0],$cycleNum-1,1) =~ m/[ACGT]/)
		{
		    $bcall[$align[3]][$cycleNum]{Count}++;

		    if(substr($align[0],$cycleNum-1,1) =~ m/A/){
			$call[$align[3]][$cycleNum]
			{substr($align[0],$cycleNum-1,1)} += $array[0];
			$call[$align[3]][$cycleNum]{CountA}++;
			if(substr($align[5],$cycleNum-1,1) !~ m/A/){
			    $bcall[$align[3]][$cycleNum]
			    {substr($align[0],$cycleNum-1,1)}++;
			    $perfect++;
			}
		    } # elsif A
		     

		    elsif (substr($align[0],$cycleNum-1,1) =~ m/C/){
			$call[$align[3]][$cycleNum]
			{substr($align[0],$cycleNum-1,1)} += $array[1];
		        $call[$align[3]][$cycleNum]{CountC}++;
			if(substr($align[5],$cycleNum-1,1) !~ m/C/){
			    $bcall[$align[3]][$cycleNum]
			    {substr($align[0],$cycleNum-1,1)}++;
			    $perfect++;
			}
			
		    } # elsif C


		    elsif (substr($align[0],$cycleNum-1,1) =~ m/G/){
			$call[$align[3]][$cycleNum]
			{substr($align[0],$cycleNum-1,1)} += $array[2];
		        $call[$align[3]][$cycleNum]{CountG}++;
			if(substr($align[5],$cycleNum-1,1) !~ m/G/){
			    $bcall[$align[3]][$cycleNum]
			    {substr($align[0],$cycleNum-1,1)}++;
			    $perfect++;
			}
			
		    } # elsif G



		    elsif (substr($align[0],$cycleNum-1,1) =~ m/T/)
		    {
			$call[$align[3]][$cycleNum]
			{substr($align[0],$cycleNum-1,1)} += $array[3];
		        $call[$align[3]][$cycleNum]{CountT}++;

			if(substr($align[5],$cycleNum-1,1) !~ m/T/){
			    $bcall[$align[3]][$cycleNum]
			    {substr($align[0],$cycleNum-1,1)}++;
			    $perfect++;
			}
			
		    } # elsif T 
		    
		} # if ACGT

		if($perfect == 0){
		    $pcall[$align[3]][$cycleNum]{Perfect0}++;
		}
		if($perfect <= 1){
		    $pcall[$align[3]][$cycleNum]{Perfect1}++;
		}
		if($perfect <= 2){
		    $pcall[$align[3]][$cycleNum]{Perfect2}++;
		}
		if($perfect <= 3){
		    $pcall[$align[3]][$cycleNum]{Perfect3}++;
		}
		$pcall[$align[3]][$cycleNum]{Count}++;
		
		if(substr($useBases,($j+1),1) =~ m/[Yy]/){
		    ## next cycle isn't a deblock (need to keep in phase)
		    ## if deblock with increment in that part of code
		    $cycleNum++;
		}
	    } # if ...
	    else 
	    {
		##  going to do a deblock 
		@array = split(' ',$sig[$j+4]);
		
		$tcall[$align[3]][$cycleNum]{A} += $array[0];
		$tcall[$align[3]][$cycleNum]{C} += $array[1];
		$tcall[$align[3]][$cycleNum]{G} += $array[2];
		$tcall[$align[3]][$cycleNum]{T} += $array[3];
		$tcall[$align[3]][$cycleNum]{Count}++;
		$cycleNum++;
	    } # else
	    
	} # for j

    } # while
    close (PIPE)
        || die "Error: failed to close pipe $sigPipe: $!";
    
    close (PIPE2)
        || die "Error: failed to close pipe $alignPipe: $!";
}

## Need to print out numbers and plot graphs
foreach $tag (sort {$a <=> $b} keys %templates)
{

    for($i=1;$i<($readLength+1);$i++)
    {
	
	print "$lane\t$i\t$templates{$tag}\t$all[$tag][$i]{Count}\t";


	## Now for All bases
	$all[$tag][$i]{Count} = 0 unless defined ($all[$tag][$i]{Count});
	foreach (@bases){

	    $all[$tag][$i]{$_} = 0 unless defined ($all[$tag][$i]{$_});

	    if ($all[$tag][$i]{Count} != 0){
		$all[$tag][$i]{$_} /= $all[$tag][$i]{Count};
	    }
	    printf ("%0.1f\t",$all[$tag][$i]{$_});
	}

	## Now for Call bases
	$call[$tag][$i]{CountA} = 0 unless defined ($call[$tag][$i]{CountA});
	$call[$tag][$i]{CountC} = 0 unless defined ($call[$tag][$i]{CountC});
	$call[$tag][$i]{CountG} = 0 unless defined ($call[$tag][$i]{CountG});
	$call[$tag][$i]{CountT} = 0 unless defined ($call[$tag][$i]{CountT});


	$call[$tag][$i]{A} = 0 unless defined ($call[$tag][$i]{A});

	if ($call[$tag][$i]{CountA} != 0){
	    $call[$tag][$i]{A} /= $call[$tag][$i]{CountA};
	}
	printf ("%0.1f\t",$call[$tag][$i]{A});


	$call[$tag][$i]{C} = 0 unless defined ($call[$tag][$i]{C});

	if ($call[$tag][$i]{CountC} != 0){
	    $call[$tag][$i]{C} /= $call[$tag][$i]{CountC};
	}
	printf ("%0.1f\t",$call[$tag][$i]{C});

	$call[$tag][$i]{G} = 0 unless defined ($call[$tag][$i]{G});

	if ($call[$tag][$i]{CountG} != 0){
	    $call[$tag][$i]{G} /= $call[$tag][$i]{CountG};
	}
	printf ("%0.1f\t",$call[$tag][$i]{G});

	$call[$tag][$i]{T} = 0 unless defined ($call[$tag][$i]{T});

	if ($call[$tag][$i]{CountT} != 0){
	    $call[$tag][$i]{T} /= $call[$tag][$i]{CountT};
	}
	printf ("%0.1f\t",$call[$tag][$i]{T});


	#### Now for the % errors

	$bcall[$tag][$i]{Count} = 0 unless defined ($bcall[$tag][$i]{Count});
	foreach (@bases){

	    $bcall[$tag][$i]{$_} = 0 unless defined ($bcall[$tag][$i]{$_});

	    if ($bcall[$tag][$i]{Count} != 0){
		$bcall[$tag][$i]{$_} /= $bcall[$tag][$i]{Count};
		$bcall[$tag][$i]{$_} *= 100;
	    }
	    printf ("%0.1f\t",$bcall[$tag][$i]{$_});
	}
	
	$bcall[$tag][$i]{N} = 0 unless defined ($bcall[$tag][$i]{N});
	if ($bcall[$tag][$i]{Count} != 0){
	    $bcall[$tag][$i]{N} /= $bcall[$tag][$i]{Count};
	    $bcall[$tag][$i]{N} *= 100;
	}
	printf ("%0.1f\t",$bcall[$tag][$i]{N});

## Now for the perfect reads
	$pcall[$tag][$i]{Count} = 0 unless defined ($pcall[$tag][$i]{Count});
	$pcall[$tag][$i]{Perfect0} = 0 unless defined ($pcall[$tag][$i]{Perfect0});
	$pcall[$tag][$i]{Perfect1} = 0 unless defined ($pcall[$tag][$i]{Perfect1});
	$pcall[$tag][$i]{Perfect2} = 0 unless defined ($pcall[$tag][$i]{Perfect2});
	$pcall[$tag][$i]{Perfect3} = 0 unless defined ($pcall[$tag][$i]{Perfect3});

	if ($pcall[$tag][$i]{Count} != 0)
	{
	    $pcall[$tag][$i]{Perfect0} 
	    = $pcall[$tag][$i]{Perfect0} / $pcall[$tag][$i]{Count};
	    $pcall[$tag][$i]{Perfect0} *= 100;
	    
	    $pcall[$tag][$i]{Perfect1} 
	    = $pcall[$tag][$i]{Perfect1} / $pcall[$tag][$i]{Count};
	    $pcall[$tag][$i]{Perfect1} *= 100;
	    
	    $pcall[$tag][$i]{Perfect2} 
	    = $pcall[$tag][$i]{Perfect2} / $pcall[$tag][$i]{Count};
	    $pcall[$tag][$i]{Perfect2} *= 100;
	    
	    $pcall[$tag][$i]{Perfect3} 
	    = $pcall[$tag][$i]{Perfect3} / $pcall[$tag][$i]{Count};
	    $pcall[$tag][$i]{Perfect3} *= 100;

	} # if

	printf ("%0.1f\t",$pcall[$tag][$i]{Perfect0});
	printf ("%0.1f\t",$pcall[$tag][$i]{Perfect1});
	printf ("%0.1f\t",$pcall[$tag][$i]{Perfect2});
	printf ("%0.1f\t",$pcall[$tag][$i]{Perfect3});

	
	## Now for Deblock bases
	$tcall[$tag][$i]{Count} = 0 unless defined ($tcall[$tag][$i]{Count});
	foreach (@bases){

	    $tcall[$tag][$i]{$_} = 0 unless defined ($tcall[$tag][$i]{$_});

	    if ($tcall[$tag][$i]{Count} != 0){
		$tcall[$tag][$i]{$_} /= $tcall[$tag][$i]{Count};
	    }
	    printf ("%0.1f\t",$tcall[$tag][$i]{$_});
	}
	
	print "\n";
    } # readLength Loop
} # lane loop

