#!/usr/bin/env perl

# PROJECT: GERALD
# MODULE:  align2qref.pl
# AUTHOR:  R. J. Shaw
#
# Copyright (c) 2008 Illumina
# This source file is covered by the "Solexa Public Source License"
# agreement and bound by the terms therein.

# Extracts reference bases from an _align.txt file, padding with `.' for
# cycles masked by USE_BASES.

use warnings;
use strict;

use FindBin;
use lib "$FindBin::Bin/../";
use Gerald::Common;

my $version;
my $id;
{
    my $cvs_id='$Id: align2qref.pl,v 1.5 2008/03/26 17:41:30 rshaw Exp $';
    my $cvs_version='$Revision: 1.5 $';
    # Make CVS tag into a displayable string
    $version=$cvs_version;
    $version=~s/\$//g;
    $version=~s/Revision//;
    $id=$cvs_id;
#    $id=~s/\$//g;
} #scope

#------------------------------------------------------------------------------

my $use_bases_opt = '--use_bases';

my $usage
    ="Usage: cat alignFile | $0 $use_bases_opt USE_BASES > qrefFile";

if (@ARGV == 1) {
    if (($ARGV[0] eq '-h') || ($ARGV[0] eq '--help')) {
	print $usage, "\n";
	exit(0);
    }
}

die $usage unless (@ARGV == 2);
die $usage unless ($ARGV[0] eq $use_bases_opt);
my $use_bases_str = $ARGV[1];

my @used_cycle_inds = parseUseBases($use_bases_str, 0, 0);
my $orig_read_length = length($use_bases_str);
my $pad_ch = '.';

my $read_seq_field_ind = 0;
my $strand_dir_field_ind = 4;
my $ref_seq_field_ind = 5;

my $not_aligned_base = '.';
my $read_length = 0;
my $not_aligned_read_ref_str = ($not_aligned_base . "\n") x $orig_read_length;
my $line_num = 0;

my $aligned = 0;
my $ref_seq;
my $padded_seq;

while (<STDIN>) {
    ++$line_num;
    chomp;

    next if (/^\#/);

    my @fields = split(/ /);

    if ($read_length == 0) {
        # Extract read length from first data line.
        $read_length = length($fields[$read_seq_field_ind]);
    }

    if ($aligned = ($#fields >= $ref_seq_field_ind)) {
        # aligned
        $ref_seq = $fields[$ref_seq_field_ind];

	# Handle match on the reverse strand.
	if ($fields[$strand_dir_field_ind] eq 'R') {
            $ref_seq = reverse($fields[$ref_seq_field_ind]);
            $ref_seq =~ tr/ACGT/TGCA/;
	}

	if (length($ref_seq) != $read_length) {
	    my $curr_ref_length = length($ref_seq);
	    die("Length of match ($curr_ref_length) in line $line_num"
		. " differs from earlier read length ($read_length)");
	}

	# Pad the reference with `.' representing cycles masked by USE_BASES.
	$padded_seq = pad_seq_with_masked_cycles($ref_seq, \@used_cycle_inds,
						 $orig_read_length, $pad_ch);
    }

    print STDOUT ($aligned
		  ? (join("\n", split(//, $padded_seq)) . "\n")
		  : $not_aligned_read_ref_str);
}

#------------------------------------------------------------------------------
