#!/usr/bin/env perl

# PROJECT: GERALD
# MODULE:  buildSeq.pl
# AUTHOR:  A. J. Cox
#
# Copyright (c) 2006 Solexa
# This source file is covered by the "Solexa Public Source License"
# agreement and bound by the terms therein.

# Build a fastq/fasta/scarf format file from a set of _seq.txt files and 
# a set of quality files

my $version;
my $id;
{
    my $cvs_id='@(#) $Id: buildSeq.pl,v 1.14 2007/10/19 15:19:13 aladdin Exp $';
    my $cvs_version='$Revision: 1.14 $';
    # Make CVS tag into a displayable string
    $version=$cvs_version;
    $version=~s/\$//g;
    $version=~s/Revision//;
    $id=$cvs_id;
#    $id=~s/\$//g;
} #scope


use warnings;
use strict;
use File::Basename;

use FindBin;
use lib "$FindBin::Bin/../";
use Gerald::Common;

my $usage=<<"END";
Usage: paste s_N_seqpre.txt [s_N_qcal.txt [s_N_filt.txt] ] | $0 
[--qualityFilter ] 
[--contamFilter ] 
[--fastq|--fasta|--scarf] 
[--numeric|--symbolic]
END

my %outputModes=
( '--fastq'=>0,
  '--fasta'=>1,
  '--scarf'=>2 );


my $qualityParam=undef;
my $contamParam=undef;
my $numericQuality=undef;


if ((@ARGV>0)&&($ARGV[0] eq '--qualityFilter'))
{
    shift (@ARGV);
    $qualityParam=1; 
} # if

if ((@ARGV>0)&&($ARGV[0] eq '--contamFilter'))
{
    shift (@ARGV);
    $contamParam=1; 
} # if

my $outputMode=$outputModes{$ARGV[0]} if (@ARGV>0);
if (defined($outputMode))
{
    shift (@ARGV);
} # if
else
{
    $outputMode=0; 
} # else

if ((@ARGV>0)&&($ARGV[0] eq '--numeric'))
{
    $numericQuality=1;
    shift(@ARGV);
} # if
elsif((@ARGV>0)&&($ARGV[0] eq '--symbolic'))
{
    shift(@ARGV);
} # elsif

die $usage unless (@ARGV==0);

# sub expandQuality
# Expand symbolic quality values to numeric ones
sub expandQuality($)
{
    my $qv=$_[0];
    my $i;
    if (defined($numericQuality))
    {
	$qv=expandSymbolicQualityString($qv);
    } # else
    return ($qv);
} # expandQuality



my $seqName;
my $seqString;
my $qualString;

if ($outputMode==0)
{ # fastq mode
    while (<STDIN>)
    { 
	chomp;
	next if ((defined($qualityParam))&&(/N$/));
	($seqName,$seqString,$qualString)=split("\t",$_);
	$qualString=expandQuality($qualString);
	print '@'."${seqName}\n${seqString}\n+${seqName}\n${qualString}\n";
    } # while
} # if
elsif ($outputMode==1)
{ # fasta mode
    while (<STDIN>)
    { # fastq mode
	chomp;
	next if ((defined($qualityParam))&&(/N$/));
	($seqName,$seqString)=split("\t",$_);
	print ">${seqName}\n${seqString}\n";
    } # while
} # elsif
elsif ($outputMode==2)
{
    while (<STDIN>)
    { # scarf mode
	chomp;
	next if ((defined($qualityParam))&&(/N$/));
	($seqName,$seqString,$qualString)=split("\t",$_);
	$qualString=expandQuality($qualString);
	print "${seqName}:${seqString}:${qualString}\n";
    } # while
} # elsif


