#!/usr/bin/env perl

use warnings;
use strict;

use FindBin;
use lib "$FindBin::Bin/../";
use Gerald::Common;

# File: plotInformation.pl
# Copyright (c) Solexa Limited 2006
# Author: A. J. Cox
# This source file is covered by the "Solexa Public Source License"
# agreement and bound by the terms therein.

# Description:
# Produces histogram of alignment score vs. num of clusters

my $version;
my $id;
{
    my $cvs_id='@(#) $Id:';
    my $cvs_version='$Revision:';
    # Make CVS tag into a displayable string
    $version=$cvs_version;
    $version=~s/\$//g;
    $version=~s/Revision//;
    $id=$cvs_id;
    $id=~s/\$//g;
} #scope


my $binSize=0.5;
my $readLength;
my @score;
my @bases=qw/A C G T/;
my @infoGainAlign;
my @numBasesAlign;
my @infoGainRealign;
my @numBasesRealign;
my @infoGainPrb;

my $priorEntropy=2;
my $toBase2=1.0/log(2.0);


sub readAlignFile
{
    my ($alignFileName, $infoRef, $numRef)=@_;
#    my ($line, $i, $readAs);
    my @a;
    my ($line, $rev, $i, $j, $k, $thisEntropy, $thisProb, $readAs, $shouldBe);

    my @allBases=();
    my @cycHash=();

    open ( FILE, $alignFileName) 
        || die "Problem opening $alignFileName: $!";

    while (<FILE>)
    {
        if (/^\#/)
        {
            next;
        } #if
        @a=split;  

        for ($i=0;$i<length($a[0]);$i++)
        {            
            $allBases[$i]{substr($a[0],$i,1)}++;
        } # for

        next if (@a==3);
        die unless (@a>=6);
        next if ($a[2]!=1);

        if ($a[4]eq'R') 
        {
            $rev=reverse($a[5]);
            $rev=~tr/ACGT/TGCA/;
        } #if
        else
        {
            $rev=$a[5];
        } #else

        
#        print "$a[0]\n$rev\n";
        for ($i=0; $i<length($a[0]); $i++)
        {
            $readAs=substr($a[0],$i,1);
            $shouldBe=substr($rev,$i,1);
            $cycHash[$i]{$readAs.$shouldBe}++;
            $cycHash[$i]{$readAs}++;
        } #for
    } #while

    # check for blank file 
    return 0 unless (defined($a[0]));

    for ($i=0; $i<length($a[0]); $i++)
    {
        for $j (@bases)
        { # work out entropy contribution for letter $j
            $allBases[$i]{$j}=0 unless defined($allBases[$i]{$j});
            $numRef->[$i]+=$allBases[$i]{$j};

            next unless 
                (defined ($cycHash[$i]{$j})&&($cycHash[$i]{$j}!=0));
            $thisEntropy=0;
            for $k (@bases)
            { # work out entropy of one instance of letter $j
                next unless 
                    (defined ($cycHash[$i]{$j.$k})&&($cycHash[$i]{$j.$k}!=0));
                $thisProb=$cycHash[$i]{$j.$k}/$cycHash[$i]{$j};
                $thisEntropy+=$thisProb*log($thisProb);
#                print "$thisProb\n";
            } # for k
            $thisEntropy*=$toBase2; # convert units to bits
            $thisEntropy+=$priorEntropy; # convert to an info gain
            $thisEntropy/=2.0; # convert units to bases from bits
#            print "$i $j $thisEntropy\n";
            $infoRef->[$i]+=$cycHash[$i]{$j}*$thisEntropy;
        } # for j
    } # for i   

    close (FILE);
} # sub readAlignFile



#my $name;
my $tmpName;
my @a;
my $i;
my $j;
#my $foundStart;
#my %hist;

#die "Usage: cat *align.txt | $0 outFile.png "
#    unless (@ARGV==1);


die "Usage: $0 out.png align.txt [realign.txt [prb.txt [useBases]]]"
    unless ((@ARGV>=2)&&(@ARGV<=5));

my ($outName, $alignFile, $realignFile, $prbFile, $useBases)= @ARGV;


readAlignFile( $alignFile, \@infoGainAlign, \@numBasesAlign);

$readLength=scalar(@numBasesAlign);

#for ($i=0; $i<$readLength; $i++) 
#{
#    print "$i $infoGainAlign[$i] $numBasesAlign[$i]\n";
#}


readAlignFile( $realignFile, \@infoGainRealign, \@numBasesRealign) 
    if (defined($realignFile));

#for ($i=0; $i<@numBasesRealign; $i++) 
#{
#    print "$i $infoGainRealign[$i] $numBasesRealign[$i]\n";
#}

if (defined($prbFile))
{

#    print "Assuming read length of $readLength\n";
    $useBases="all" unless defined ($useBases);
    if ($useBases eq "all")
    {
        $useBases= "Y" x $readLength;
    } # if
    elsif ($useBases eq "odd")
    {
        $useBases= "YN" x ($readLength);
        $useBases=substr($useBases,0,length($useBases)-1);
    } # elsif
    else
    {
        die "Use bases string $useBases must be odd, all or YNYN..."
            unless ($useBases=~/^[YNyn]+$/);
        $useBases = uc ($useBases);
    } # else

    my @ub=split(//,$useBases);
    my @baseIndex;
    for ($i=0;$i<@ub;$i++)
    {
	push (@baseIndex, $i) if ($ub[$i] eq "Y"); 
    } # if

    die "Need $readLength bases of base information, only ", 
    scalar(@baseIndex), " specified in $useBases"
	unless (scalar(@baseIndex)>=$readLength);
    @baseIndex=splice(@baseIndex,0,$readLength);

    my $i;
    my @aa;
    my $thisProb;
    my $thisQual;
    my $thisEntropy;
    my $couldntFind=0;

    open ( FILE, $prbFile) 
        || die "Problem opening $prbFile: $!";

    

    while (<FILE>)
    {
        chomp;
        next if (/^\#/);
        @a=split('\t');
        for ($i=0; $i<@baseIndex; $i++)
        {
                @aa=split(' ',$a[$baseIndex[$i]]);
#                warn unless (@aa==4);
		warn "Failed to find data for cycle ", 
		$i+1, " in file $prbFile"
		    if ((@aa!=4)&&($couldntFind++==0));


                $thisEntropy=0;
                for $thisQual (@aa)
                {
                    # convert quality value to prob
                    $thisProb=1.0/(1.0+10**(-$thisQual/10.0));
                    $thisEntropy=$thisProb*log($thisProb);             
                } # for
                $thisEntropy*=$toBase2; # units to bits
                $thisEntropy+=$priorEntropy; # change to info gain
                $thisEntropy/=2.0; # units from bits to bases
                $infoGainPrb[$i]+=$thisEntropy;
        } # for
    } # while
    close (FILE);
} # if


$tmpName="/tmp/tmp-plotInformation-$>-$$";


#for (sort {$a<=>$b} keys %hist1)
#{
#    print "$_\t$hist1{$_}\n";
#}

my $plotCommand.="plot '$tmpName.dat' using 1:2 title 'raw bases' with lines lt 7 lw 24";

if (scalar(@infoGainPrb) != 0 )
{

    $plotCommand.=", '$tmpName.dat' using 1:6 title 'quality info' with lines lt 3 lw 24";        
} # if


$plotCommand.=", '$tmpName.dat' using 1:3 title 'raw info' with lines lt 1 lw 24";        


if (scalar(@infoGainRealign) != 0 )
{

#    $plotCommand.=", '$tmpName.dat' using 1:4 title 'realign bases' with lines lt 2 lw 24";        
    $plotCommand.=", '$tmpName.dat' using 1:5 title 'realign info' with lines lt 2 lw 24";        
} # if





#print "$minBin $maxBin\n";

open (OUT, ">$tmpName.dat")
	|| die "Problem opening $tmpName.dat for writing: $!";

for (my $i=0; $i<@infoGainAlign; $i++)
{
    $infoGainAlign[$i]=0 unless (defined($infoGainAlign[$i]));
    $infoGainRealign[$i]=0 unless (defined($infoGainRealign[$i]));
    $numBasesRealign[$i]=0 unless (defined($numBasesRealign[$i]));
    print OUT $i+1; 
    print OUT " $numBasesAlign[$i] $infoGainAlign[$i]"; 
    print OUT " $numBasesRealign[$i] $infoGainRealign[$i]"
        if (scalar(@infoGainRealign)!=0);
    print OUT " $infoGainPrb[$i]"
        if (scalar(@infoGainPrb)!=0);
    print OUT "\n";
} # for

close (OUT);

open (OUT, ">${tmpName}.txt") 
    || die "Problem opening $tmpName.txt for writing: $!";

print OUT << "END";

    set terminal postscript portrait color solid;
#    set nokey;

    set xrange[1:$readLength];
    set xtics 1;
    set yrange [0:12000];
    set ytics 1000;

    set size 1.2, 1.0;

    set title "Information content for tile $outName";
    set xlabel "Cycle";
    set ylabel "Bases";

    $plotCommand

END
 
    close (OUT);
    gnuplotImage($tmpName.".txt", $outName);
    unlink "$tmpName.dat", "$tmpName.txt";

