#!/usr/bin/env perl

# File: plotIntensity_for_IVC.pl
# Copyright (c) 2006 Solexa
# Author: A. J. Cox, L. J. Davies
# This source file is covered by the "Illumina Genome Analyzer
# Software License Agreement" and bound by the terms therein.

# Description:
# Produces intensity cycle by cycle plot for lanes 

use warnings;
use strict;

use FindBin;
use File::Spec;
use lib "$FindBin::Bin/../";
use Gerald::Common;

my $version;
my $id;
{
    my $cvs_id='@(#) $Id: plotIntensity_for_IVC.pl,v 1.10 2008/05/15 16:00:01 mzerara Exp $';
    my $cvs_version='$Revision: 1.10 $';
    # Make CVS tag into a displayable string
    $version=$cvs_version;
    $version=~s/\$//g;
    $version=~s/Revision//;
    $id=$cvs_id;
    $id=~s/\$//g;
} #scope


my $maxNumLanes=8;

sub getTempName() {
    return "/tmp/tmp-plotIntensity-$>-$$";
}

my $linewidth=3;

sub draw_graphs {
    my ($lane,$inputName,$outName,$title,$max) = @_;
    
    my $tmpName = getTempName().".txt";

    open (OUT, ">$tmpName") 
	|| die "Problem opening $tmpName for writing: $!";

    print OUT << "END";
set terminal postscript portrait color solid;
set yrange [0:$max];
set ytics 500;
set size 1.0, 1.0;
set origin 0.0, 0.0;
set title "$title";
set xlabel "Cycle Number";
set ylabel "Intensity";
plot '$inputName' using 1:2 title 'A' with linespoints lt 9 lw $linewidth pt 7 ps 2, '$inputName' using 1:3 title 'C' with linespoints lt 1 lw $linewidth pt 7 ps 2,    '$inputName' using 1:4 title 'G' with linespoints lt 3 lw $linewidth pt 7 ps 2, '$inputName' using 1:5 title 'T' with linespoints lt 2 lw $linewidth pt 7 ps 2;

END
    close (OUT);
    gnuplotImage($tmpName, $outName);
    unlink $tmpName;
}

sub draw_graphs_percent {
    my ($lane,$inputName,$outName,$title) = @_;

    my $tmpName = getTempName().".txt";

    open (OUT, ">$tmpName") 
	|| die "Problem opening $tmpName for writing: $!";

    print OUT << "END";
set terminal postscript portrait color solid;
set yrange [0:100];
set ytics 10;
set size 1.0, 1.0;
set origin 0.0, 0.0;
set title "$title";
set xlabel "Cycle Number";
set ylabel "Percent";
  plot '$inputName' using 1:2 title 'A' with linespoints lt 9 lw $linewidth pt 7 ps 2, '$inputName' using 1:3 title 'C' with linespoints lt 1 lw $linewidth pt 7 ps 2,    '$inputName' using 1:4 title 'G' with linespoints lt 3 lw $linewidth pt 7 ps 2, '$inputName' using 1:5 title 'T' with linespoints lt 2 lw $linewidth pt 7 ps 2;

END
    close (OUT);
    gnuplotImage($tmpName, $outName);
    unlink $tmpName;
}



### MAIN
die "Usage: $0 Signal_Means.txt Directory_Name\n"
    unless (@ARGV==2);

my $file      = shift(@ARGV);
my $directory = shift(@ARGV);

my @intensities;

open (IN, $file) 
    || die "Problem opening $file: $!";
while (<IN>)
{
  
    next if($_ =~ m/\#/);
    push (@intensities,[split('\t',$_)]);

}
close(IN);


my %lanes;
for(my $i=0;$i<scalar(@intensities);$i++){
    $lanes{$intensities[$i][0]}++;
}


### Going to plot graphs now


### Going to do CALL 
### Going to do call stuff first as will be highest intensities

my $max    = 0;
my $setmax = 0;

my $lane;
for($lane=1; $lane<=$maxNumLanes; $lane++){
    
    if (exists $lanes{$lane}){

	my $fileName = "s_".$lane."_call.png";
	my $outName  = File::Spec->catfile($directory, $fileName);
	print "$outName\n";

	my $tmpName=getTempName().".dat";
	open (OUT, ">$tmpName")
	    || die "Problem opening $tmpName for writing: $!";

	for(my $j=0;$j<scalar(@intensities);$j++){
	    next if ($intensities[$j][0] != $lane);

	    print  OUT "$intensities[$j][1]\t";
	    for(my $k=0;$k<4;$k++){
	 
		print  OUT "$intensities[$j][$k+6]\t";
		if($setmax == 0){
		
		    ### going to defined a max value based on first tile
		    if($intensities[$j][$k+6] > $max){
			$max = $intensities[$j][$k+6];
		    }	
		} 
	    }
	    print OUT "\n";
	}
	close (OUT);

	if($setmax == 0){
	    $max = $max * 1.5;
	    ## wanted to round to nearest 1000 so will cheat until i have more time
	    $max = $max / 1000;
	    $max = int($max);
	    $max = ($max+1) * 1000;
	    $setmax++;
	}
	draw_graphs($lane,$tmpName,$outName,
		    "Called intensities lane ".$lane, $max);
	unlink $tmpName;
    }
}


### Going to do ALL 
for($lane=1; $lane<=$maxNumLanes; $lane++){
    
    if (exists $lanes{$lane}){
	my $fileName = "s_".$lane."_all.png";
	my $outName  = File::Spec->catfile($directory, $fileName);
	print "$outName\n";

	my $tmpName=getTempName().".dat";
	open (OUT, ">$tmpName")
	    || die "Problem opening $tmpName for writing: $!";

	for(my $j=0;$j<scalar(@intensities);$j++){
	    next if ($intensities[$j][0] != $lane);

	    for(my $k=0;$k<5;$k++){
		print  OUT "$intensities[$j][$k+1]\t";
	    }
	    print OUT "\n";
	}
	close (OUT);
	draw_graphs($lane,$tmpName,$outName,
		    "All intensities lane ".$lane, $max);
	unlink $tmpName;
    }
}


### Going to do %ALL 
for($lane=1; $lane<=$maxNumLanes; $lane++){
   
    if (exists $lanes{$lane}){

	my $fileName = "s_".$lane."_percent_all.png";
	my $outName  = File::Spec->catfile($directory, $fileName);
	print "$outName\n";

	my $tmpName=getTempName().".dat";
	open (OUT, ">$tmpName")
	    || die "Problem opening $tmpName for writing: $!";

	for(my $j=0;$j<scalar(@intensities);$j++){
	    next if ($intensities[$j][0] != $lane);
	    my $total = 0;
	    for(my $k=0;$k<4;$k++){
		$total+= $intensities[$j][$k+2];
	    }
	    print  OUT "$intensities[$j][1]\t";
	    for(my $k=0;$k<4;$k++){
		if($total != 0){
		    $intensities[$j][$k+2] =  $intensities[$j][$k+2] / $total * 100;
		}
		else {
		    $intensities[$j][$k+2] = 0;
		}   
		print  OUT "$intensities[$j][$k+2]\t";
	    }
	    print OUT "\n";
	}
	close (OUT);

	draw_graphs_percent($lane,$tmpName,$outName,
			    "Percent All intensities lane ".$lane);
	unlink $tmpName;
    }
}


### Going to do %Call 
for($lane=1; $lane<=$maxNumLanes; $lane++){
    
    if (exists $lanes{$lane}){

	my $fileName = "s_".$lane."_percent_call.png";
	my $outName  = File::Spec->catfile($directory, $fileName);
	print "$outName\n";

	my $tmpName=getTempName().".dat";
	open (OUT, ">$tmpName")
	    || die "Problem opening $tmpName for writing: $!";

	for(my $j=0;$j<scalar(@intensities);$j++){
	    next if ($intensities[$j][0] != $lane);
	    my $total = 0;
	    for(my $k=0;$k<4;$k++){
		$total+= $intensities[$j][$k+6];
	    }
	    print  OUT "$intensities[$j][1]\t";
	    for(my $k=0;$k<4;$k++){
		if($total != 0){
		    $intensities[$j][$k+6] =  $intensities[$j][$k+6] / $total * 100;
		}
		else {
		    $intensities[$j][$k+6] =  0;
		}	
		print  OUT "$intensities[$j][$k+6]\t";
	    }
	    print OUT "\n";
	}
	close (OUT);
	draw_graphs_percent($lane,$tmpName,$outName,
			    "Percent Called intensities lane ".$lane);
	unlink $tmpName;
    }
}


### Going to do %Base Calls
for($lane=1; $lane<=$maxNumLanes; $lane++){
    
    if (exists $lanes{$lane}){

	my $fileName = "s_".$lane."_percent_base.png";
	my $outName  = File::Spec->catfile($directory, $fileName);
	print "$outName\n";

	my $tmpName=getTempName().".dat";
	open (OUT, ">$tmpName")
	    || die "Problem opening $tmpName for writing: $!";
    
	for(my $j=0;$j<scalar(@intensities);$j++){
	    next if ($intensities[$j][0] != $lane);

	    print  OUT "$intensities[$j][1]\t";
	    for(my $k=0;$k<4;$k++){
		print  OUT "$intensities[$j][$k+10]\t";
	    }
	    print OUT "\n";
	}
	close (OUT);

	draw_graphs_percent($lane,$tmpName,$outName,
			    "Percent Basecalls lane ".$lane);
	unlink $tmpName;
    }
}
