#!/usr/bin/env perl

# PROJECT: GERALD
# MODULE:  reportQuality.pl
# AUTHOR:  A. J. Cox
#
# Copyright (c) 2006 Solexa
# This source file is covered by the "Solexa Public Source License"
# agreement and bound by the terms therein.

# Simple script to evaluate some quality value stats on a set of tiles

my $version;
my $id;
{
    my $cvs_id='@(#) $Id: reportQuality.pl,v 1.8 2007/06/12 15:27:34 tc Exp $';
    my $cvs_version='$Revision: 1.8 $';
    # Make CVS tag into a displayable string
    $version=$cvs_version;
    $version=~s/\$//g;
    $version=~s/Revision//;
    $id=$cvs_id;
#    $id=~s/\$//g;
} #scope

print "#RUN_TIME ". scalar(localtime)."\n";
print "#SOFTWARE_VERSION $id\n";

use warnings;
use strict;
use File::Basename;

my $usage="Usage: $0 [-qualityFilter filterPath filterParam] [-contamFilter contamParam] [-qhgPath qhgPath] readLength qualitySuffix [-qualityFormat symbolic|numeric] alignFile[s]";

die $usage unless (@ARGV>=3);

#my $useBases=undef;
my $qualityParam=undef;
my $qualityFilter=undef;
my $contamParam=undef;
my $qhgPath=undef;

#if ($ARGV[0] eq '-useBases')
#{
#    shift (@ARGV);
#    $useBases=shift(@ARGV); 
#} # if

if ($ARGV[0] eq '-qualityFilter')
{
    shift (@ARGV);
    $qualityFilter=shift(@ARGV);
    die "Error: could not find quality filter executable $qualityFilter" 
	unless (-e $qualityFilter);
    die "Error: file $qualityFilter is not executable" 
	unless (-x $qualityFilter);
    $qualityParam=shift(@ARGV); 
} # if

if ($ARGV[0] eq '-contamFilter')
{
    shift (@ARGV);
    $contamParam=shift(@ARGV); 
} # if

if ($ARGV[0] eq '-qhgPath')
{
    shift (@ARGV);
    $qhgPath=shift(@ARGV); 
} # if

die $usage unless (@ARGV>=3);

my $readLength=shift(@ARGV);

die "Specified read length of $readLength, need positive value"
    unless ($readLength>0);

#my $exptDir=shift (@ARGV);

#die "Error: could not find experiment directory $exptDir" 
#    unless (-e $exptDir);
#die "Error: $exptDir is not a directory" 
#    unless (-d $exptDir);

my $qualitySuffix=shift (@ARGV);
my $isNumeric=0;

if (($ARGV[0] eq '--symbolic')||($ARGV[0] eq '--numeric'))
{
    $isNumeric=1 if ($ARGV[0] eq '--numeric');
    shift (@ARGV);
} # if


my $file;
my $stem;
my $type;
my $pipe;
my $dir;
my $line;
my $inten;
my @s0;
my @s1;
my @s2;
my @s3;


my @baseIndex;
#my $readLength=25;
#$useBases="NYYYYYYYYYYYYYYYYYYYYYYYYYYY" unless defined($useBases);

my %total;
my %error;
my $i;
my $j;
my $qv;
my $numBases=0;

my $alignFileName;


sub printLine
{
    my ($qv, $etot, $btot,$numBases)=@_;

    # % error expected if the quality values are 100% accurate
    my $expectedError=100/(1+10**($qv/10));

    printf "$qv\t$etot\t$btot\t%5.2f\t%5.2f\t|\t$error{$_}\t$total{$_}\t%5.2f\t%5.2f\t|\t%5.3f\n",
    100*$etot/$btot, 100.0*$btot/$numBases, 
    100*$error{$_}/$total{$_}, 100*$total{$_}/$numBases, $expectedError;
#    die;
} # sub printLine


#print scalar(@baseIndex), " $baseIndex[0] $baseIndex[-1]\n"; die;

my $finished=0;
my $doQuality=0;
my $numTiles;
my $tileList;

do
{
    %total=();
    %error=();
    $numBases=0;

    for $file (@ARGV)
    {
	$numTiles++;
	$dir=dirname($file);
	if (!defined($qhgPath)) {
	    $qhgPath=$dir;
	}
#    print dirname($file), "\n";
	unless (basename($file)=~/(s_[1-8]_[0-9][0-9][0-9][0-9])_(.*\.txt)$/)
	{
	    die "Could not parse tile from file name $file";
	} # unless
	$stem=$1;
	$type=$2;
	$tileList.=$stem;
	$tileList.=((($numTiles%8)==0)?"\n":" ");

	$alignFileName="${dir}/${stem}_align.txt";

	unless (-f $alignFileName)
	{
	    print STDERR basename($0), 
	    ": Error - could not find file $alignFileName\n";
	    exit (0);
	} # unless

	$pipe = "cat ${dir}/${stem}_align.txt | grep -v '^\#' | paste - ${stem}${qualitySuffix} |";
	$pipe .= " ${qualityFilter} '${qualityParam}' ${qhgPath}/${stem}_qhg.txt |"
	    if ($doQuality); 

	if (defined($contamParam)) 
	{
	    if (defined($qualityParam)) 
	    {        
		$pipe .= " qualityFilter.pl '${contamParam}' ${dir}/${stem}_crediff.txt |";
	    } # if
	    else
	    {
		$pipe .= " qualityFilter.pl '${contamParam}' ${dir}/${stem}_cprediff.txt |";
	    } # else
	} # if
	open (LINE, $pipe) || die "Failed to open $pipe";
	while (<LINE>)
	{ 

	    @s0=split('\t',$_);
	    @s2=split(' ',shift(@s0));
	    next if (scalar(@s2)==3);
	    if ($s2[4] eq 'R')
	    {
		$s2[5]=reverse($s2[5]);
		$s2[5]=~tr/ACGTacgt/TGCAtgca/;
	    } # if
	    
	    if ($isNumeric)
	    {
		@s1=split(' ',shift(@s0));
	    }
	    else
	    {
		@s1=();
		for ($i=0;$i<length($s0[0]);$i++)
		{
		    push @s1, ord(substr($s0[0],$i,1))-64;
		}
	    }

	    for ($i=0;$i<$readLength;$i++)
	    {
#            print "$i $baseIndex[$i] $s1[$baseIndex[$i]]\n";

		unless (defined($s1[$i]))
		{
		    print STDERR basename($0), ": Error parsing line: $_";
		    print STDERR basename($0),
		    ": can't find quality values for cycle ",
		    $i+1, " of tile ${stem} ";
			print STDERR 
			    "- only ",scalar(@s1), 
			    " cycles of quality values present on line\n";
		    exit (-1);
		}
		$qv=$s1[$i];
		$numBases++;
		$total{$qv}++;
		unless (defined (substr($s2[0],$i,1))&&defined(substr($s2[5],$i,1)))
{
    print "$_\n";
}
		$error{$qv}+=(substr($s2[0],$i,1) ne substr ($s2[5],$i,1));
	    } # for i

	} # while  

    } # for file

    unless ($doQuality)
    {
	print " ** Quality value report **\n\n";
	print "Tiles analysed:\n", $tileList;
	print "\n$numTiles tiles in all\n\n"; 
	print " ** Results for raw data **\n"; 
    } # unless
    else
    {
	print "\n\n\nResults for data filtered on $qualityParam\n"; 
    } # else

    my $esofar=0;
    my $tsofar=0;
    
    print "-\t-\t-\t-\t-\t-\t-\t-\t-\t-\t-\t-\n";
    print "QV\tthis QV or higher\t\t|\tthis QV only\t\t\t|\texpected\n";
    print "\t# error\t# bases\t% error\t% total\t|\t# error\t# bases\t% error\t% total\t|\t% error\n";
    print "-\t-\t-\t-\t-\t-\t-\t-\t-\t-\t-\t-\n";

    for (sort {$b<=>$a} keys %total )
    {
	$esofar+=$error{$_};
	$tsofar+=$total{$_};
	$error{$_}=0 unless defined($error{$_});
	
	printLine($_,$esofar,$tsofar,$numBases);
    } # for

    unless (defined($qualityParam))
    {
	$finished=1;
    } # unless
    elsif ($doQuality==0)
    {
	$doQuality=1;
    } # elsif
    else
    {
	$finished=1;
    } # else

} # do
while ($finished==0);



