#!/usr/bin/env python

# Copyright (c) 2005, 2006 Solexa
# Author: Klaus Maisinger
# This source file is covered by the "Solexa Public Source License"
# agreement and bound by the terms therein.

import re, os, os.path

"""Filename conventions for the Swerdlotron prototype instrument."""

class SwerdlotronFileName:
    expr = re.compile("([0-9]*)_ch([0-9])_cyc([0-9][0-9]*)_(\D{1,6})\.(\D{1,3})")
    def match(filename):
        b = SwerdlotronFileName.expr.match(os.path.basename(filename))
        if b:
            return [ b.group(1), b.group(2), b.group(3), b.group(4) ]
        return None
    match = staticmethod(match)

    def instrument():
        return "Swerdlotron"
    instrument = staticmethod(instrument)

    def cmp(name1, name2):
        c=cmp(int(SwerdlotronFileName.cycle(name1)),
              int(SwerdlotronFileName.cycle(name2)))
        if c != 0:
            return c
        c=cmp(int(SwerdlotronFileName.is_deblocked(name1)),
              int(SwerdlotronFileName.is_deblocked(name2)))
        if c != 0:
            return c
        bases = [ "C", "A", "T", "G"]
        idx1 = bases.index(SwerdlotronFileName.base(name1))
        idx2 = bases.index(SwerdlotronFileName.base(name2))
        
        return cmp(idx1, idx2)
    cmp = staticmethod(cmp)

    def is_deblocked(filename):
        idx = int(SwerdlotronFileName.match(filename)[2])-1
        return idx%2
    is_deblocked = staticmethod(is_deblocked)


    def tile(filename):
        return 0
    tile = staticmethod(tile)

    def lane(filename):
        return (int(SwerdlotronFileName.match(filename)[1]) - 1)
    lane = staticmethod(lane)

    def cycle(filename):
        idx = int(SwerdlotronFileName.match(filename)[2])-1
        return idx/2+1
    cycle = staticmethod(cycle)

    def tilestr(filename):
        match = SwerdlotronFileName.match(filename)
        return "_".join([match[0], match[1]])
    tilestr = staticmethod(tilestr)

    def base(filename):
        bases = { "red" : "C",
                  "orange": "A",
                  "green": "T",
                  "blue" : "G" }
        return bases[SwerdlotronFileName.match(filename)[3]]
    base = staticmethod(base)


if __name__ == "__main__":
    names = sys.argv[1:]
    for name in names:
        print name
        print SwerdlotronFileName.match(name)
        print SwerdlotronFileName.tile(name)
        print SwerdlotronFileName.lane(name)
        print SwerdlotronFileName.cycle(name)
        print SwerdlotronFileName.is_deblocked(name)
        print SwerdlotronFileName.tilestr(name)
        print SwerdlotronFileName.base(name)
        print
       
