#ifndef QHG_PARAMETERS
#define QHG_PARAMETERS
/*
 * File: QhgParameters.hh
 * Copyright (c) Illumina 2008
 * Author:       Come Raczy
 * This source file is covered by the "Solexa Public Source License"
 * agreement and bound by the terms therein.
 */

#include <vector>
#include "PositionIndex.h"
//#include "ClusterNeighborhoodsBuilder.hh"
#include "ClusterLocation.hh"

/*
 * Class containing the quality parameters used by quahog.
 */
struct QhgParameters
{
  QhgParameters() {} 
  QhgParameters
  ( const float        * ints,
    const int            numCycles,
    const int            numChannels,
    const uint           pureBases,
    const std::vector<bool> & basesToUse ); 
  // Mike's purity definition: largest intensity as a fraction of total
  float purity;
  // My purity definition: largest as fraction of (largest + next largest)
  float chastity;
  // Similarity (cosine) with the intensity vectors of neighbour clusters
  float similarity;
  uint distance;
  std::vector<int> neighbours;
  std::vector<std::vector<float> > intensity;
};

/**************************************************
 * Class QhgParametersBuilder
 * Builds the QhgParameters for a set of clusters
 **************************************************/
class QhgParametersBuilder
{
    static const int neighborhoodRadius;
    // cell dimensions of the grid used to bin the clusters to speedup the neighgorhood search
    static const int binSizeX;
    static const int binSizeY;
public:
    QhgParametersBuilder(const std::vector<bool> &basesToUse,
                         const unsigned int pureBases,
                          std::vector<ClusterLocation<int> > &clusterLocationList,
                         const std::vector<float * >   intensitiesList,
                         const unsigned int numberOfClusters,
                         const unsigned int numberOfChannels);
 
    inline const std::vector<QhgParameters> &getQhgParametersList() {return qhgParametersList_;}
private:
    std::vector<QhgParameters> qhgParametersList_;
    const unsigned int numberOfClusters_; 
    const unsigned int numberOfCycles_;
    const unsigned int numberOfChannels_;
    void initializeQhgParametersList(const std::vector<bool> &basesToUse,
                                     const unsigned int pureBases,
                                     const std::vector<float * >   intensitiesList);
    void assignNeighborhood(const ClusterLocation<int> &cluster, const std::vector<ClusterLocation<int> > &neighborhood);
};

int writeQhgFile(const char                    filename[],
                 const PositionIndex  * clusterPos,
                 const std::vector<QhgParameters> & clusterPurity,
                 const int                     numClusters);
  
#endif // #ifndef QHG_PARAMETERS
