#include <utility>

#include "SignalMeansTile.hh"
#include "Legacy.hh"

using namespace std;
unsigned int SignalMeans::numberOfChannels = 4;
int signalMeansTile(std::vector<SignalMeans> &signalMeansList,
                    int &numberOfValidClusters,
                    std::vector<float *> intensities,  
                    std::vector<unsigned char> &sequenceList,
                    std::vector<QhgParameters> qhgParametersList,
                    const QhgFilter &qhgFilter)
{
    const std::string validBases = "ACGT";
    const unsigned int numberOfClusters = qhgParametersList.size();
    const unsigned int numberOfCycles = intensities.size();
    std::vector<std::vector<float> > channelSumList(numberOfCycles, std::vector<float>(SignalMeans::numberOfChannels, 0.0));
    typedef std::pair<float, unsigned int> CallData;
    std::vector<std::vector<CallData> > callDataList(numberOfCycles, std::vector<CallData>(4, CallData(0.0, 0)));
    numberOfValidClusters = 0;
    for (unsigned int cluster = 0; cluster < numberOfClusters; ++cluster)
    {
        if (qhgFilter(qhgParametersList[cluster]))
        {
            numberOfValidClusters++;
            // Note: bases are organised by clusters first (cycle 1 for all clusters, then cycle 2 for all clusters, ...)
            vector<unsigned char>::const_iterator currentBase = sequenceList.begin() + cluster;
            for (unsigned int cycle = 0; cycle < numberOfCycles; ++cycle)
            {
                for (unsigned int channel = 0; channel < SignalMeans::numberOfChannels; ++channel)
                {
                    // LEGACY: tu get the exact same results as in the 0.3 release, the values are rounded and truncated
                    // in the same way as they used to be in the intermediary text file
                    channelSumList[cycle][channel] += Legacy::truncate<1>(intensities[cycle][cluster*SignalMeans::numberOfChannels + channel]);
                }
                const size_t base = validBases.find(*currentBase);
                if (base != validBases.npos)
                {
                    // This WILL break if the number of channels is not 4 (requires a channel-to-base mapping)
                    callDataList[cycle][base].first += intensities[cycle][cluster*SignalMeans::numberOfChannels + base];
                    callDataList[cycle][base].second++;
                }
                currentBase += numberOfClusters;
            }
        }
    }
    signalMeansList.resize(numberOfCycles);
    for(unsigned int cycle = 0; cycle < numberOfCycles; ++cycle)
    {
        SignalMeans &signalMeans = signalMeansList[cycle];
        for (unsigned int channel = 0; channel < SignalMeans::numberOfChannels; ++channel)
        {
            signalMeans.channelMeanList[channel] = numberOfValidClusters ? channelSumList[cycle][channel] / numberOfValidClusters : 0.0;
        }
        for (unsigned int base = 0; base < 4; ++base)
        {
            const int callCount = callDataList[cycle][base].second;
            signalMeans.callCountList[base] = callCount;
            signalMeans.callMeanList[base] = callCount ? callDataList[cycle][base].first / callCount : 0.0;
        }
    }
    return 0;
}


int writeSignalMeansTile(FILE *fh, const std::vector<SignalMeans> &signalMeansList,
                         const int totalNumberOfClusters, const int numberOfValidClusters, const int lane)
{
    if (!fh)
    {
        return 1;
    }
    fprintf(fh, "# Clusters: Filtered %d Original %d\n", numberOfValidClusters, totalNumberOfClusters);
    fprintf(fh, "#\n");
    fprintf(fh, "#Lane\tCycle\tAll A\tAll C\tAll G\tAll T\tCall A\tCall C\tCall G\tCall T\tNum A\tNum C\tNum G\tNum T\tNum X\n");
    for (unsigned int cycle = 0; cycle < signalMeansList.size(); cycle++)
    {

        fprintf(fh, "%d\t%d\t", lane, cycle+1);
        for (unsigned int channel = 0; channel  < SignalMeans::numberOfChannels; ++channel)
        {
            fprintf(fh, "%0.1f\t", signalMeansList[cycle].channelMeanList[channel]);
        }
        for (unsigned int base = 0; base  < 4; ++base)
        {
            fprintf(fh, "%0.1f\t", signalMeansList[cycle].callMeanList[base]);
        }
        int invalidCalls = numberOfValidClusters;
        for (unsigned int base = 0; base  < 4; ++base)
        {
            invalidCalls -= signalMeansList[cycle].callCountList[base];
            fprintf(fh, "%d\t", signalMeansList[cycle].callCountList[base]);
        }
        fprintf(fh, "%d\n", invalidCalls);
    }
    return 0;
}

