#include <vector>
#include <set>
#include <string>
#include <iostream>
#include <fstream>
#include <sstream>
#include <stdio.h>

using namespace std;

struct Cluster
{
    Cluster(int xx, int yy, int i, vector<int> n) : x(xx), y(yy), id(i), neighborhood(n) {}
    int x;
    int y;
    int id;
    vector<int> neighborhood;
};

ostream &operator<<(ostream &os, const Cluster &c)
{
    os << c.id << ":" << c.x << ", " << c.y << ":";
    for (vector<int>::const_iterator i = c.neighborhood.begin(); i != c.neighborhood.end(); ++i)
    {
        os << ":" << *i;
    }
    return os;
}

int main(int argc, char *argv[])
{
    const string programName = argv[0];
    if (argc != 3)
    {
        cout << "usage: " << programName << " <qhg file> <radius>" << endl;
        exit(1);
    }
    const string qhgFile = argv[1];
    string radiusString = string(argv[2]);
    istringstream iradius(radiusString);
    int radius;
    iradius >> radius;
    ifstream is;
    is.open(qhgFile.c_str());
    //const int bufferSize = 100000;
    //char buffer[bufferSize];
    int lineNumber = 0;
    int id = 0;
    vector<Cluster> clusterList;
    while(is)
    {
        ++lineNumber;
        int lane, tile, x, y;
	float a,b,c,d;
        is >> lane >> tile >> x >> y >> a >> b >> c >> d;
        string neighborhood;
        is >> neighborhood;
        //cout << lane << "\t"  << tile << "\t" << x << "\t" << y << "\t" << a << "\t" << b << "\t" << c << "\t" << d << "\t" << neighborhood << endl;
	vector<int> neighborList;
        istringstream tmp(neighborhood);
        ostringstream found;
        while (tmp) 
        {
            int n;
            char c;
            tmp >> n >> c;
            neighborList.push_back(n);
            if (found.str().size() > 0)
                found << ':';
            found << n;
        }
        if (neighborhood.compare(found.str()))
            cout << "line " << lineNumber << ": expected: " << neighborhood << ", got : " << found.str() << " (probably no end line on last line)" << endl;
        clusterList.push_back(Cluster(x, y, id, neighborList));
        ++id;
    }
    is.close();
    for (vector<Cluster>::const_iterator cluster = clusterList.begin(); cluster != clusterList.end(); ++cluster)
    {
        // check that all neighbors are actually within the radius and
        // check that there are no repetitions within the neighborhood and
        // check that no other cluster  should be in the radius and
        // check that the neighbors are sorted by distance to the center.
        int distance = 0;
        for (vector<int>::const_iterator i = cluster->neighborhood.begin(); i != cluster->neighborhood.end(); ++i)
        {
            if (0 > *i)
            {
                if (-1 > *i || cluster->neighborhood.size() != 1)
                {
                    cout << "invalid neighbor: " << *cluster << endl;
                }
                else
                {
                    break;
                }
            }
            if (*i == cluster - clusterList.begin())
            {
                cerr << "cluster " << *cluster << " is in its own neighbor"<< endl;
            }
            const int dx = cluster->x - clusterList[*i].x;
            const int dy = cluster->y - clusterList[*i].y;
            if (dx*dx + dy*dy < distance)
            {
                cout << "cluster " << *cluster << ": neighbors are not sorted by increasing distance" << endl;
            }
            if (dx*dx + dy*dy == distance && i != cluster->neighborhood.begin() && *i < *(i-1))
            {
                cout << "cluster " << *cluster << ": equidistant neighbors are not sorted by increasing id" << endl;
            }
            distance = dx*dx + dy*dy;
            if (dx*dx + dy*dy > radius*radius)
            {
                cerr << "cluster " << *cluster << " ---- " << clusterList[*i] << " is out of radius (" << dx*dx + dy*dy << " > " <<  radius*radius << ")" << endl;
            }
            for (vector<int>::const_iterator j = i+1; j != cluster->neighborhood.end(); ++j)
            {
                if (*j == *i)
                {
                    cout << "cluster " << *cluster << ": repetitions in the neighborhood" << endl;
                }
            }
            vector<int> n;
            for (vector<Cluster>::const_iterator k = clusterList.begin(); k != clusterList.end(); ++k)
            {
                if (k == cluster) continue;
                const int dx = cluster->x - k->x;
                const int dy = cluster->y - k->y;
                if (dx*dx + dy*dy < radius*radius)
                    n.push_back(k-clusterList.begin());
            }
            // remove repetitions in the original cluster neighborhood
            set<int> sref(cluster->neighborhood.begin(), cluster->neighborhood.end());
            vector<int> nref(sref.begin(), sref.end());
            if (n.size() != nref.size())
            {
                cerr << "cluster " << *cluster << " should have these neighbors";
                for (vector<int>::const_iterator l = n.begin(); l != n.end(); ++l) cerr << ':' << *l;
                cerr << endl; 
            }
        }
    }
}
