#include <cstdio>
#include <vector>

using namespace std;

enum ExportFileFieldName
{
  Machine=0,
  RunNumber,
  Lane, 
  Tile, 
  XCoordinate,
  YCoordinate,
  IndexString,
  ReadNumber,
  Read, 
  QualityString,
  MatchChromosome,
  MatchContig,
  MatchPosition,
  MatchStrand,
  MatchDescriptor,
  SingleReadAlignmentScore,
  PairedReadAlignmentScore,
  PartnerChromosome,
  PartnerContig,
  PartnerOffset,
  PartnerStrand,
  Filtering,
  NumberOfEntries // not a field, but if you stick it 
  // on the end of the list of fields it automatically gets right value
};

class ExportFileReaderImp;
// ExportFileReader
class ExportFileReader
{
 public:
  ExportFileReader(ExportFileReaderImp* p): p_(p) {} 

  ~ExportFileReader() {}

  // Returns pointer to ASCII sequence of next oligo, or null if at end
  bool getNextEntry( void ) ;

  // Rewind - next oligo read will be first in list
  void rewind ( void );

  const char* getMachine( void ) const; 
  const int getRunNumber( void ) const; 
  const int getLane( void ) const; 
  const int getTile( void ) const; 
  const int getXCoordinate( void ) const;
  const int getYCoordinate( void ) const;
  const char* getIndexString( void ) const;
  const int getReadNumber( void ) const;
  const char* getRead ( void ) const;
  const char* getQualityString( void ) const;
  const char* getMatchChromosome( void ) const;
  const char* getMatchContig( void ) const;
  const int getMatchPosition( void ) const;
  const char getMatchStrand( void ) const;
  const char* getMatchDescriptor( void ) const;
  const int getSingleReadAlignmentScore( void ) const;
  const int getPairedReadAlignmentScore( void ) const;
  const char* getPartnerChromosome( void ) const;
  const char* getPartnerContig( void ) const;
  int getPartnerOffset( void ) const;
  char getPartnerStrand( void ) const;
  char getFiltering( void ) const;
 protected:
  ExportFileReaderImp* p_;
}; // ~class ExportFile

class ExportFileReaderImp
{
 public:

  virtual ~ExportFileReaderImp() {}

  // Returns pointer to ASCII sequence of next oligo, or null if at end
  virtual bool getNextEntry( void ) =0;

  // Rewind - next oligo read will be first in list
  virtual void rewind ( void ) =0;

  virtual const char* getMachine( void ) const=0; 
  virtual const int getRunNumber( void ) const=0; 
  virtual const int getLane( void ) const=0; 
  virtual const int getTile( void ) const=0; 
  virtual const int getXCoordinate( void ) const=0;
  virtual const int getYCoordinate( void ) const=0;
  virtual const char* getIndexString( void ) const=0;
  virtual const int getReadNumber( void ) const=0;
  virtual const char* getRead ( void ) const=0;
  virtual const char* getQualityString( void ) const=0;
  virtual const char* getMatchChromosome( void ) const=0;
  virtual const char* getMatchContig( void ) const=0;
  virtual const int getMatchPosition( void ) const=0;
  virtual const char getMatchStrand( void ) const=0;
  virtual const char* getMatchDescriptor( void ) const=0;
  virtual const int getSingleReadAlignmentScore( void ) const=0;
  virtual const int getPairedReadAlignmentScore( void ) const=0;
  virtual const char* getPartnerChromosome( void ) const=0;
  virtual const char* getPartnerContig( void ) const=0;
  virtual const int getPartnerOffset( void ) const=0;
  virtual const char getPartnerStrand( void ) const=0;
  virtual const char getFiltering( void ) const=0;
 protected:
}; // ~class ExportFileReaderImp

class ExportFileReaderActual : public ExportFileReaderImp
{
 public:
  ExportFileReaderActual( const char* exportFileName );


  virtual ~ExportFileReaderActual();

  // Returns pointer to ASCII sequence of next oligo, or null if at end
  virtual bool getNextEntry( void );

  // Rewind - next oligo read will be first in list
  virtual void rewind ( void );

  virtual const char* getMachine( void ) const; 
  virtual const int getRunNumber( void ) const; 
  virtual const int getLane( void ) const; 
  virtual const int getTile( void ) const; 
  virtual const int getXCoordinate( void ) const;
  virtual const int getYCoordinate( void ) const;
  virtual const char* getIndexString( void ) const;
  virtual const int getReadNumber( void ) const;
  virtual const char* getRead ( void ) const;
  virtual const char* getQualityString( void ) const;
  virtual const char* getMatchChromosome( void ) const;
  virtual const char* getMatchContig( void ) const;
  virtual const int getMatchPosition( void ) const;
  virtual const char getMatchStrand( void ) const;
  virtual const char* getMatchDescriptor( void ) const;
  virtual const int getSingleReadAlignmentScore( void ) const;
  virtual const int getPairedReadAlignmentScore( void ) const;
  virtual const char* getPartnerChromosome( void ) const;
  virtual const char* getPartnerContig( void ) const;
  virtual const int getPartnerOffset( void ) const;
  virtual const char getPartnerStrand( void ) const;
  virtual const char getFiltering( void ) const;
 protected:
  char buf_[120000];
  vector<const char*> entry_;
  FILE* pFile_;


}; // ~class ExportFileReaderActual


