// -*- c++ -*-
/*****************************************************************************/
// Copyright (c) Illumina 2007
// Author: Richard Shaw
// This source file is covered by the "Solexa Public Source License"
// agreement and bound by the terms therein.
/*****************************************************************************/
//
// This class can be used to read parameter value data from a file in chunks,
// each of which will be returned as a DataSet object.
//
// The file format consists of a variable number of columns containing the
// parameters. Each row corresponds to one base-call. The final two columns
// contain the called and aligned base respectively.
//
/*****************************************************************************/

#ifndef DATA_SET_FACTORY_H
#define DATA_SET_FACTORY_H

/*****************************************************************************/

#include <string>

#include "File_Buffer.h"
#include "ReadParams.h"

/*****************************************************************************/

class Data_Set_Factory
{
 public:
    Data_Set_Factory(const std::string& qval_file_path_str,
                     File_Buffer::Compression_Type qval_file_compr,
                     const std::string& qref_file_path_str = "",
                     File_Buffer::Compression_Type qref_file_compr
                     = File_Buffer::compressionNone);
    ~Data_Set_Factory();

    /** This method allocates the DataSet. Caller must delete it.
        Only DataSet::MAX_NUM_RECORDS file lines will be parsed per call.
    */
    bool get_next_data_set(DataSet*& data_set_ptr, bool& at_eof);

 private:
    File_Buffer my_qval_file_buffer;
    File_Buffer my_qref_file_buffer;
    bool my_need_qref_file_flag;

    bool read_ref_base(char& ref_base, bool& at_eof);
};

/*****************************************************************************/

#endif // ! DATA_SET_FACTORY_H

/*****************************************************************************/
