/*
 * Copyright (c) Solexa 2006
 * Author: Klaus Maisinger
 * This source file is covered by the "Solexa Public Source License"
 * agreement and bound by the terms therein.
 */

#include <iostream>
#include "DataTable.h"

DataTable::DataTable()
{
}

DataTable::~DataTable()
{
}

void DataTable::create()
{
    BinnedTable<ErrorLookup<ErrorCount> >::generateTable();
    int i;
    for (i = 0; i < table.size(); i++)
        table[i].resize(4*4);
}

void DataTable::print()
{
    long sum_count = 0;
    long sum_error = 0;
    int i, j, k;
    for (i = 0; i < table.size(); i++) {
        std::vector<float> params = getBinParams(i);
        for (j = 0; j < table[i].size(); j++) {
            printf("%6i %3i:% 9li %9li (", i, j,
                   table[i][j].count, table[i][j].error); 
            for (k = 0; k < params.size(); k++)
                printf(" %6.1f", params[k]);
            printf(")\n");
            sum_count += table[i][j].count;
            sum_error += table[i][j].error;
        }
    }
    printf("%6i %3i: %9li %9li\n",
           table.size(), 16, sum_count, sum_error);
}

/* 
 * Bin the input data.
 */
void DataTable::addData(DataSet &data_set)
{
    if ((data_set.getNumberParameters() == 0) ||
        (data_set.getNumberValues() == 0) ||
        (binnings.size() == 0))
        return;
    if (data_set.getNumberParameters() != binnings.size()) {
        std::cerr << "Wrong number of parameters in data set ("
            << data_set.getNumberParameters() << " instead of "
            << binnings.size() << "). Ignoring data set." << std::endl;
        return;
    }    
    if (table.size() == 0)
        create();
    int i, j;
    int no_params = binnings.size();
    std::vector<float> params(no_params);
    std::vector<std::vector<float> >  *values = data_set.getParameterValues();
    std::vector<BaseAlignment>        *bases = data_set.getBaseReads();
 
    for (i = 0; i < no_params; i++) {
        assert(values->at(i).size() == bases->size());
    }
    for (j = 0; j < bases->size(); j++) {
        for (i = 0; i < no_params; i++)
            params[i] = values->at(i)[j];
        int idx1 = getBinIndex(params);
#if 0
        std::vector<int> indices = getBinIndices(params);
        printf("idx1 %i:", idx1);
        printf("%f %f %f ", params[0], params[1], params[2]);
        printf(" %i %i, %i %i, %i %i\n",
               indices[0], binnings[0].size(),
               indices[1], binnings[1].size(),
               indices[2], binnings[2].size());
#endif
               
        int idx2 = table[idx1].getErrorIndex(bases->at(j));
        if (idx2 < 0)
            continue;
        table[idx1][idx2].count += 1;
        if (bases->at(j).base_aligned != bases->at(j).base_read)
            table[idx1][idx2].error += 1;
    }
}
