/*
 * Copyright (c) Solexa 2006
 * Author: Klaus Maisinger
 * This source file is covered by the "Solexa Public Source License"
 * agreement and bound by the terms therein.
 */

/*
 * This class stores a data set read from a file by Data_Set_Factory.
 */
#ifndef READPARAMS_H_
#define READPARAMS_H_

#include <vector>

class Data_Set_Factory;

class BaseAlignment {
public:
    BaseAlignment() : base_read(0), base_aligned(0) {};
    char base_read;
    char base_aligned;
};

class DataSet {
    friend class Data_Set_Factory;

    std::vector<std::vector<float> > values;
    std::vector<BaseAlignment> reads;
public:
    DataSet();
    virtual ~DataSet();
    int getNumberParameters() const;
    int getNumberValues() const;
    void addParameterValues(std::vector<float> &new_values);
    std::vector<float> *getParameterValues(int idx);
    std::vector<std::vector<float> > *getParameterValues();
    std::vector<BaseAlignment> *getBaseReads();
    void clear();

 private:
    enum { ALLOC_SIZE = 100000, MAX_NUM_RECORDS = 10000000 };

    unsigned int my_num_params;
    unsigned int my_num_records;
};
#endif /*READPARAMS_H_*/
