/*****************************************************************************/
// Copyright (c) Illumina 2008
// Author: Richard Shaw
// This source file is covered by the Illumina Genome Analyzer Software 
// License Agreement and bound by the terms therein.
/*****************************************************************************/

#include <string>
#include <map>
#include <list>
#include <iostream>

#include <cstring>

#include "option_test_utils.h"
#include "StringUtil.h"

#include "QApply_Options.h"

/*****************************************************************************/

int main()
{
    QApply_Options qapply_options;

    const std::string expected_app_name("QualityApply");
    const std::string qtable_path_str("s_3_qtable.txt");
    const std::string qval_path1_str("s_3_0059_qval.txt");
    const std::string qval_path2_str("s_3_0060_qval.txt");

    const std::string no_arg_str("NOARG");

    Opt_Arg_Map opt_arg_map;
    opt_arg_map["orig_read_lengths"] = "36:36";
    opt_arg_map["qval_compression"] = "gzip";
    opt_arg_map["numeric"] = no_arg_str;

    Arg_List extra_arg_list;
    extra_arg_list.push_back(qtable_path_str);
    extra_arg_list.push_back(qval_path1_str);
    extra_arg_list.push_back(qval_path2_str);


    // Count NOARGs.

    int num_no_args(0);

    for (Opt_Arg_Map_Iter opt_arg_iter = opt_arg_map.begin();
         opt_arg_iter != opt_arg_map.end(); ++opt_arg_iter) {
        if (opt_arg_iter->second == no_arg_str) {
            ++num_no_args;
        }
    }

    int num_args(1 + (opt_arg_map.size() * 2) - num_no_args
                 + extra_arg_list.size());


    // Populate arg_cstr_arr.
    // (Note that the cstrs have to be non-const to match the signature
    //  of getopt_long as used by CommandLine::Options.)

    char** arg_cstr_arr = new char*[num_args + 1];
    *(arg_cstr_arr + num_args) = 0;

    int arg_ind(0);
    arg_cstr_arr[arg_ind++] = copy_to_new_cstr(expected_app_name.c_str());

    for (Opt_Arg_Map_Iter opt_arg_iter = opt_arg_map.begin();
         opt_arg_iter != opt_arg_map.end(); ++opt_arg_iter) {
        arg_cstr_arr[arg_ind++] = copy_to_new_cstr(std::string("--") +
                                                   opt_arg_iter->first);
        if (opt_arg_iter->second != no_arg_str) {
            arg_cstr_arr[arg_ind++] = copy_to_new_cstr(opt_arg_iter->second);
        }
    }

    for (Arg_List_Iter arg_iter = extra_arg_list.begin();
         arg_iter != extra_arg_list.end(); ++arg_iter) {
        arg_cstr_arr[arg_ind++] = copy_to_new_cstr(*arg_iter);
    }

    // Dump arg_cstr_arr.
    std::cout << "Arguments tested (" << num_args << ") :-" << std::endl;

    for (arg_ind = 0; arg_ind < num_args; ++arg_ind) {
        std::cout << arg_cstr_arr[arg_ind] << std::endl;
    }

    qapply_options.init(num_args, arg_cstr_arr);


    // Tidy up arg_cstr_arr after use.
    for (arg_ind = 0; arg_ind < num_args; ++arg_ind) {
        delete[](arg_cstr_arr[arg_ind]);
    }


    // Check the values parsed.

    bool args_ok(true);

    args_ok = args_ok && (qapply_options.my_num_cycles == 72);
    args_ok = 
        args_ok && check_opt_arg_match(opt_arg_map, "qval_compression",
                                       qapply_options.my_qval_compr_name);
    args_ok = args_ok && (qapply_options.my_numeric_output_flag == true);
    args_ok = args_ok && (qapply_options.my_qtable_path_str
                          == qtable_path_str);
    args_ok = args_ok && (qapply_options.my_qval_path_str_list.size() == 2);

    QApply_Options::Path_Str_List_CIter qval_path_str_citer
        = qapply_options.my_qval_path_str_list.begin();

    args_ok = args_ok && (*qval_path_str_citer++ == qval_path1_str);
    args_ok = args_ok && (*qval_path_str_citer++ == qval_path2_str);

    if (args_ok) {
        std::cout << "Arguments were correctly parsed." << std::endl;
    }

    return 0;
}

/*****************************************************************************/
