/*****************************************************************************/
// Copyright (c) Illumina 2007
// Author: Richard Shaw
// This source file is covered by the "Solexa Public Source License"
// agreement and bound by the terms therein.
/*****************************************************************************/

#include <iostream>
#include <cstdio>

#include "XMLConfigHandler.h"
#include "XMLSAXParser.h"

enum { ERR_OK=0, ERR_ARGS, ERR_FILE, ERR_XML_PARSE };

/*****************************************************************************/

int usage(const std::string app_name, int ret_val)
{
    std::cerr << "Usage: " << app_name << " <config_file.xml>" << std::endl;
    return ret_val;
}

/*****************************************************************************/

int main(int num_args, char* arg_cstr_arr[])
{
    // Check args.

    const std::string app_name(arg_cstr_arr[0]);

    if (num_args != 2) {
        return usage(app_name, ERR_ARGS);
    }

    
    // Open file.

    const std::string cfg_file_path_str(arg_cstr_arr[1]);

    FILE* cfg_file_hndl = fopen(cfg_file_path_str.c_str(), "r");

    if (!cfg_file_hndl) {
        return usage(app_name, ERR_FILE);
    }

    // Parse file.

    XMLConfigHandler cfg_handler;
    XMLSAXParser* sax_parser_ptr = new XMLSAXParser();
    sax_parser_ptr->setContentHandler(&cfg_handler);
    sax_parser_ptr->parse(cfg_file_hndl);
    fclose(cfg_file_hndl);

    Parameter_Config_List param_cfg_list;

    if (!cfg_handler.get_param_cfg_list(param_cfg_list)) {
        std::cerr << "Failed to extract parameter configurations from "
                  << cfg_file_path_str << std::endl;
        return ERR_XML_PARSE;
    }

    // Dump parameter configurations.

    std::cout << "Extracted the following parameter configurations :-"
              << std::endl;

    for (Parameter_Config_List_CIter param_cfg_citer = param_cfg_list.begin();
         param_cfg_citer != param_cfg_list.end(); ++param_cfg_citer) {
        std::cout << *param_cfg_citer << std::endl;
    }

    return ERR_OK;
}

/*****************************************************************************/

